package org.jboss.jsr299.tck.tests.context;

import java.lang.annotation.Annotation;

import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.context.ContextNotActiveException;
import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.context.SessionScoped;
import javax.enterprise.context.spi.Context;
import javax.enterprise.context.spi.Contextual;
import javax.enterprise.context.spi.CreationalContext;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
public class ContextTest extends AbstractJSR299Test
{

   private @interface Dummy
   {

   }

   private @interface Unregistered
   {

   }

   private static class DummyContext implements Context
   {

      public <T> T get(Contextual<T> bean, CreationalContext<T> creationalContext)
      {
         throw new UnsupportedOperationException();
      }

      public <T> T get(Contextual<T> contextual)
      {
         return get(contextual, null);
      }

      public Class<? extends Annotation> getScopeType()
      {
         return Dummy.class;
      }

      public boolean isActive()
      {
         return true;
      }
   }

   @Test(expectedExceptions = { IllegalStateException.class }, groups = { "manager" })
   @SpecAssertion(section = "6.5.1", id = "b")
   public void testGetContextWithTooManyActiveContextsFails()
   {
      Context firstContext = new DummyContext()
      {
      };
      Context secondContext = new DummyContext()
      {
      };
      getCurrentManager().addContext(firstContext);
      getCurrentManager().addContext(secondContext);
      getCurrentManager().getContext(Dummy.class);
   }

   @Test(expectedExceptions = { ContextNotActiveException.class }, groups = { "manager" })
   @SpecAssertion(section = "6.5.1", id = "a")
   public void testGetContextWithNoRegisteredContextsFails()
   {
      getCurrentManager().getContext(Unregistered.class);
   }

   @Test(groups = { "contexts" })
   @SpecAssertions({
      @SpecAssertion(section = "6.2", id = "aa"),
      @SpecAssertion(section = "2.4.1", id = "a"),
      @SpecAssertion(section = "2.4.1", id = "b"),
      @SpecAssertion(section = "2.4.1", id = "c"),
      @SpecAssertion(section = "11.3.18", id = "a")
   })
   public void testBuiltInContexts()
   {
      Context context = getCurrentManager().getContext(Dependent.class);
      assert context != null;
      context = getCurrentManager().getContext(RequestScoped.class);
      assert context != null;
      context = getCurrentManager().getContext(SessionScoped.class);
      assert context != null;
      context = getCurrentManager().getContext(ApplicationScoped.class);
      assert context != null;
   }
}
