package org.jboss.jsr299.tck.tests.context;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.context.spi.Context;
import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.impl.MockCreationalContext;
import org.jboss.jsr299.tck.tests.context.dependent.ejb.FoxRunLocal;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * This class tests a basic context against section 8 of the specification 
 * 
 * Spec version: 20090519
 * 
 * @author Nicklas Karlsson
 * @author Pete Muir
 * @author David Allen
 */
@Artifact
public class NormalContextTest extends AbstractJSR299Test
{

   @Test(groups = { "contexts" })
   @SpecAssertions( { 
      @SpecAssertion(section = "6.2", id = "j"), 
      @SpecAssertion(section = "6.3", id = "c") 
   } )
   public void testGetReturnsExistingInstance()
   {
      Bean<MySessionBean> mySessionBean = getBeans(MySessionBean.class).iterator().next();
      CreationalContext<MySessionBean> creationalContext = getCurrentManager().createCreationalContext(mySessionBean);
      MySessionBean first = getCurrentManager().getContext(SessionScoped.class).get(mySessionBean, creationalContext);
      first.setId(10);
      MySessionBean second = getCurrentManager().getContext(SessionScoped.class).get(mySessionBean, creationalContext);
      assert second.getId() == 10;
   }

   @Test(groups = { "contexts" })
   @SpecAssertions( { 
      @SpecAssertion(section = "6.2", id = "l")
   } )
   public void testGetWithCreationalContextReturnsNewInstance()
   {
      MyContextual bean = new MyContextual(getCurrentManager());
      bean.setShouldReturnNullInstances(false);
      getCurrentManager().addBean(bean);

      CreationalContext<MySessionBean> creationalContext = new MockCreationalContext<MySessionBean>();
      MySessionBean newBean = getCurrentManager().getContext(SessionScoped.class).get(bean, creationalContext);
      assert newBean != null;
      assert bean.isCreateCalled();
   }

   @Test(groups = { "contexts" })
   @SpecAssertion(section = "6.2", id = "nb")
   public void testGetMayNotReturnNullUnlessContextualCreateReturnsNull()
   {
      // The case of no creational context is already tested where a null is
      // returned. Here we just test that the contextual create can return null.
      MyContextual bean = new MyContextual(getCurrentManager());
      bean.setShouldReturnNullInstances(true);
      getCurrentManager().addBean(bean);

      CreationalContext<MySessionBean> creationalContext = new MockCreationalContext<MySessionBean>();
      assert getCurrentManager().getContext(SessionScoped.class).get(bean, creationalContext) == null;
      assert bean.isCreateCalled();
   }

   @Test(groups = { "contexts" })
   @SpecAssertions( { 
      @SpecAssertion(section = "6.2", id = "p"),
      @SpecAssertion(section = "6.3", id = "d")
   })
   public void testContextDestroysBeansWhenDestroyed()
   {
      MyContextual bean = new MyContextual(getCurrentManager());
      bean.setShouldReturnNullInstances(false);
      getCurrentManager().addBean(bean);

      Context sessionContext = getCurrentManager().getContext(SessionScoped.class);
      CreationalContext<MySessionBean> creationalContext = getCurrentManager().createCreationalContext(bean);
      MySessionBean instance = sessionContext.get(bean, creationalContext);
      instance.ping();
      assert instance != null;
      assert bean.isCreateCalled();
      
      destroyContext(sessionContext);
      assert bean.isDestroyCalled();
   }
   
   @Test(groups = { "contexts" })
   @SpecAssertions( { 
      @SpecAssertion(section = "6.3", id = "e")
   })
   public void testSameNormalScopeBeanInjectedEverywhere()
   {
      SimpleBeanA instanceOfA = getInstanceByType(SimpleBeanA.class);
      SimpleBeanB instanceOfB = getInstanceByType(SimpleBeanB.class);
      assert instanceOfA.getZ() == instanceOfB.getZ();
   }
}
