package org.jboss.jsr299.tck.tests.context.application.ejb;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.testng.annotations.Test;

/**
 * EJB and related tests with the built-in application context.
 * 
 * @author David Allen
 *
 * Spec version: 20090519
 */
@Artifact
@IntegrationTest
@Packaging(PackagingType.EAR)
public class EJBApplicationContextTest extends AbstractJSR299Test
{
   @Test(groups = { "broken", "contexts", "webservice", "integration" })
   @SpecAssertion(section = "6.7.3", id = "b")
   public void testApplicationScopeActiveDuringWebSericeInvocation()
   {
      //TODO This test might work better with a separate client over HTTP
      FeederService birdFeeder = getInstanceByType(FeederService.class);
      Bird bird = getInstanceByType(Bird.class);
      bird.eat();
      assert birdFeeder.adequateFood();
   }

   @Test(groups = { "contexts", "ejb3", "integration" })
   @SpecAssertion(section = "6.7.3", id = "dc")
   public void testApplicationScopeActiveDuringCallToEjbTimeoutMethod() throws Exception
   {
      FMS flightManagementSystem = getInstanceByType(FMS.class);
      flightManagementSystem.climb();
      Thread.sleep(250);
      assert flightManagementSystem.isApplicationScopeActive();
   }

   @Test(groups = { "contexts", "ejb3", "integration" })
   @SpecAssertion(section = "6.7.3", id = "ef")
   public void testApplicationContextSharedBetweenEJBTimeoutInvocations() throws Exception
   {
      FMS flightManagementSystem = getInstanceByType(FMS.class);
      flightManagementSystem.climb();
      flightManagementSystem.descend();
      Thread.sleep(250);
      assert flightManagementSystem.isSameBean();
   }

}
