/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.jsr299.tck.tests.context.dependent.ejb;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@IntegrationTest
@Packaging(PackagingType.EAR)
@Artifact
public class DependentContextEjbTest extends AbstractJSR299Test
{
   @Test(groups = { "contexts", "injection", "ejb3", "integration" })
   //@SpecAssertion(section = "8.3", id = "k")
   @SpecAssertion(section = "unknown", id = "unknown")
   public void testContextIsActiveDuringEJBDependencyInjection()
   {
      Bean<FoxRunLocal> foxRunBean = getBeans(FoxRunLocal.class).iterator().next();
      CreationalContext<FoxRunLocal> creationalContext = getCurrentManager().createCreationalContext(foxRunBean);
      FoxRunLocal foxRun = foxRunBean.create(creationalContext);
      assert foxRun.getFox() != null;
   }

   @Test(groups = { "contexts", "postconstruct", "ejb3", "integration" })
   @SpecAssertion(section = "6.4.1", id = "ka")
   public void testContextIsActiveDuringEJBPostConstruct()
   {
      Fox.setDependentContextActiveDuringPostConstruct(false);
      getInstanceByType(FoxLocal.class).getName();
      assert Fox.isDependentContextActiveDuringPostConstruct();
   }

   @Test(groups = { "contexts", "ejb3", "integration", "ri-broken"})
   @SpecAssertion(section = "6.4.3", id = "baa")
   public void testDestroyingEjbDestroysDependents() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getBeans(HouseLocal.class).size() == 1;
            Bean<HouseLocal> bean = getBeans(HouseLocal.class).iterator().next();
            CreationalContext<HouseLocal> creationalContext = getCurrentManager().createCreationalContext(bean);
            HouseLocal instance = bean.create(creationalContext);
            Room.destroyed = false;
            Table.destroyed = false;
            bean.destroy(instance, creationalContext);
            assert Room.destroyed;
            assert Table.destroyed;
         }

      }.run();
   }
   
   @Test(groups = { "contexts", "ejb3", "integration","ri-broken", "broken" })
   @SpecAssertion(section = "6.4.3", id = "baa")
   public void testDestroyingEjbDestroysDependentSimples() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getBeans(FarmLocal.class).size() == 1;
            Bean<FarmLocal> farmBean = getBeans(FarmLocal.class).iterator().next();
            CreationalContext<FarmLocal> creationalContext = getCurrentManager().createCreationalContext(farmBean);
            FarmLocal farm = farmBean.create(creationalContext);
            Horse.destroyed = false;
            Stable.destroyed = false;
            farmBean.destroy(farm, creationalContext);
            assert Horse.destroyed;
            assert Stable.destroyed;
         }

      }.run();
   }

}
