/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.jsr299.tck.tests.decorators.invocation;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * @author pmuir
 *
 */
@Artifact
@BeansXml("beans.xml")
public class DecoratorInvocationTest extends AbstractJSR299Test
{

   @Test
   @SpecAssertions({
      @SpecAssertion(section="8.5", id="a"),
      @SpecAssertion(section="8.5", id="c"),
      @SpecAssertion(section="8.5", id="b"),
      @SpecAssertion(section="8.1.3", id="d"),
      @SpecAssertion(section="8.1.2", id="f"),
      @SpecAssertion(section="8.1.2", id="b")
   })
   public void testDecoratorInvocation()
   {
      TimestampLogger.setMessage(null);
      MockLogger.setMessage(null);
      getInstanceByType(CowShed.class).milk();
      assert TimestampLogger.getMessage().equals(CowShed.MESSAGE);
      assert MockLogger.getMessage().equals(CowShed.MESSAGE);
   }
   
   @Test
   @SpecAssertions({
      @SpecAssertion(section="8.5", id="d"),
      @SpecAssertion(section="8.5", id="e"),
      @SpecAssertion(section="8.5", id="f"),
      @SpecAssertion(section="8.4", id="a"),
      @SpecAssertion(section="8.1.3", id="d"),
      @SpecAssertion(section="8.1.2", id="f"),
      @SpecAssertion(section="8.1.2", id="b")
   })
   public void testChainedDecoratorInvocation()
   {
      FooDecorator1.setMessage(null);
      FooDecorator2.setMessage(null);
      FooImpl.setMessage(null);
      getInstanceByType(CowShed.class).washDown();
      assert FooDecorator1.getMessage().equals(CowShed.MESSAGE);
      assert FooDecorator2.getMessage().equals(CowShed.MESSAGE + FooDecorator1.SUFFIX);
      assert FooImpl.getMessage().equals(CowShed.MESSAGE + FooDecorator1.SUFFIX + FooDecorator2.SUFFIX);
   }

}
