package org.jboss.jsr299.tck.tests.definition.binding;

import java.lang.annotation.Annotation;
import java.util.Set;

import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.InjectionPoint;

import org.apache.commons.httpclient.methods.GetMethod;
import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.AnyLiteral;
import org.jboss.jsr299.tck.literals.CurrentLiteral;
import org.jboss.jsr299.tck.literals.NewLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
public class BindingDefinitionTest extends AbstractJSR299Test
{
   @Test
   @SpecAssertions({
      @SpecAssertion(section = "2.3.1", id = "a0"),
      @SpecAssertion(section = "2.3.1", id = "aa")
   })
   public void testDefaultBindingDeclaredInJava()
   {
      Bean<Order> order = getBeans(Order.class).iterator().next();
      assert order.getBindings().size() == 2;
      assert order.getBindings().contains(new CurrentLiteral());
      assert order.getBindings().contains(new AnyLiteral());
   }
   
   @Test
   @SpecAssertions({
      @SpecAssertion(section = "2.3.1", id = "b"),
      @SpecAssertion(section = "11.1", id = "c")
   })
   public void testDefaultBindingForInjectionPoint()
   {
      Bean<Order> order = getBeans(Order.class).iterator().next();
      InjectionPoint injectionPoint = order.getInjectionPoints().iterator().next();
      assert injectionPoint.getBean().getBindings().contains(new CurrentLiteral());
   }
   
   @Test
   @SpecAssertion(section = "2.3.1", id = "a0")
   public void testNewBindingAndAnyBindingMutualExclusive()
   {
      Bean<OrderProcessor> order = getBeans(OrderProcessor.class, new NewLiteral()).iterator().next();
      assert order.getBindings().size() == 1;
      assert order.getBindings().iterator().next().equals(new NewLiteral());
   }

   @Test(groups = { "annotationDefinition", "rewrite" })
   @SpecAssertion(section = "2.3.2", id = "ba")
   public void testBindingDeclaresBindingAnnotation()
   {
      // Probably can use new SPI for this...
      assert !getBeans(Tarantula.class, new TameBinding()).isEmpty();
   }

   @Test
   @SpecAssertion(section = "2.3.3", id = "a")
   public void testBindingsDeclaredInJava()
   {
      Bean<Cat> cat = getBeans(Cat.class, new SynchronousBinding()).iterator().next();
      assert cat.getBindings().size() == 2;
      assert cat.getBindings().contains(new SynchronousBinding());
   }

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "2.3.3", id = "d")
   })
   public void testMultipleBindings()
   {
      Bean<?> model = getBeans(Cod.class, new ChunkyBinding(true), new WhitefishBinding()).iterator().next();
      assert model.getBindings().size() == 3;
   }

   @Test(groups = { "injection", "producerMethod" })
   @SpecAssertion(section = "2.3.5", id = "a")
   public void testFieldInjectedFromProducerMethod() throws Exception
   {
	  Bean<Barn> barnBean = getBeans(Barn.class).iterator().next(); 
      Barn barn = barnBean.create(getCurrentManager().createCreationalContext(barnBean));
      assert barn.petSpider != null;
      assert barn.petSpider instanceof DefangedTarantula;
   }

   @Test
   @SpecAssertion(section = "4.1", id = "aa")
   public void testBindingDeclaredInheritedIsInherited() throws Exception
   {
      Set<? extends Annotation> bindings = getBeans(BorderCollie.class, new HairyBinding(false)).iterator().next().getBindings();
      assert bindings.size() == 2;
      assert bindings.contains(new HairyBinding(false));
      assert bindings.contains(new AnyLiteral());
   }

   @Test
   @SpecAssertion(section = "4.1", id = "aa")
   public void testBindingNotDeclaredInheritedIsNotInherited()
   {
      Set<? extends Annotation> bindings = getBeans(ShetlandPony.class).iterator().next().getBindings();
      assert bindings.size() == 2;
      assert bindings.contains(new CurrentLiteral());
      assert bindings.contains(new AnyLiteral());
   }

   @Test
   @SpecAssertion(section = "4.1", id = "aa")
   public void testBindingDeclaredInheritedIsBlockedByIntermediateClass()
   {
      Set<? extends Annotation> bindings = getBeans(ClippedBorderCollie.class, new HairyBinding(true)).iterator().next().getBindings();
      assert bindings.size() == 2;
      Annotation hairyLiteral = new HairyBinding(true);
      assert bindings.contains(hairyLiteral);
      assert bindings.contains(new AnyLiteral());
   }
   
   @Test
   @SpecAssertion(section = "4.1", id = "ag")
   public void testBindingDeclaredInheritedIsIndirectlyInherited()
   {
      Set<? extends Annotation> bindings = getBeans(EnglishBorderCollie.class, new HairyBinding(false)).iterator().next().getBindings();
      assert bindings.size() == 2;
      assert bindings.contains(new HairyBinding(false));
   }

}
