package org.jboss.jsr299.tck.tests.deployment.lifecycle;

import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.event.Observes;
import javax.enterprise.inject.Current;
import javax.enterprise.inject.spi.AfterBeanDiscovery;
import javax.enterprise.inject.spi.AfterDeploymentValidation;
import javax.enterprise.inject.spi.BeanManager;

class ManagerObserver
{
   private static boolean managerInitialized = false;
   private static boolean managerDeployed = false;
   private static boolean requestContextActive = false;
   private static boolean applicationContextActive = false;
   
   public void managerInitialized(@Observes AfterBeanDiscovery event, @Current BeanManager beanManager)
   {
      managerInitialized = true;
      requestContextActive = beanManager.getContext(RequestScoped.class).isActive();
      applicationContextActive = beanManager.getContext(ApplicationScoped.class).isActive();
   }

   public void managerDeployed(@Observes AfterDeploymentValidation event, @Current BeanManager beanManager)
   {
      assert managerInitialized : "Manager should have been initialized before deployed";
      managerDeployed = true;
      requestContextActive &= beanManager.getContext(RequestScoped.class).isActive();
      applicationContextActive &= beanManager.getContext(ApplicationScoped.class).isActive();
   }

   public static boolean isManagerInitialized()
   {
      return managerInitialized;
   }

   public static void setManagerInitialized(boolean managerInitialized)
   {
      ManagerObserver.managerInitialized = managerInitialized;
   }

   public static boolean isManagerDeployed()
   {
      return managerDeployed;
   }

   public static void setManagerDeployed(boolean managerDeployed)
   {
      ManagerObserver.managerDeployed = managerDeployed;
   }

   public static boolean isRequestContextActive()
   {
      return requestContextActive;
   }

   public static void setRequestContextActive(boolean requestContextActive)
   {
      ManagerObserver.requestContextActive = requestContextActive;
   }

   public static boolean isApplicationContextActive()
   {
      return applicationContextActive;
   }

   public static void setApplicationContextActive(boolean applicationContextActive)
   {
      ManagerObserver.applicationContextActive = applicationContextActive;
   }
}
