package org.jboss.jsr299.tck.tests.event.implicit;

import javax.enterprise.context.Dependent;
import javax.enterprise.event.Event;
import javax.enterprise.inject.TypeLiteral;
import javax.enterprise.inject.deployment.Standard;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.AnyLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Test that there is an implicit event bean for every legal event type
 * 
 * Spec version: 20090519
 * 
 * @author Dan Allen
 */
@Artifact
public class ImplicitEventTest extends AbstractJSR299Test
{
   @Test
   @SpecAssertions({
      @SpecAssertion(section = "10.4.2", id = "a"),
      @SpecAssertion(section = "10.4.2", id = "b"),
      @SpecAssertion(section = "10.1", id = "i")
   })
   public void testImplicitEventExistsForEachEventType()
   {
      assert getBeans(new TypeLiteral<Event<StudentRegisteredEvent>>() {}, new AnyLiteral()).size() == 1;
      assert getBeans(new TypeLiteral<Event<CourseFullEvent>>() {}, new AnyLiteral()).size() == 1;
      assert getBeans(new TypeLiteral<Event<AwardEvent>>() {}, new AnyLiteral()).size() == 1;
      assert getBeans(new TypeLiteral<Event<AwardEvent>>() {}, new AnyLiteral(), new HonorsLiteral()).size() == 1;
   }
   
   @Test
   @SpecAssertions({
      @SpecAssertion(section = "10.4.2", id = "c"),
      @SpecAssertion(section = "10.4.2", id = "d"),
      @SpecAssertion(section = "10.4.2", id = "e"),
      @SpecAssertion(section = "10.4.2", id = "g"),
      @SpecAssertion(section = "6.6.2", id = "e")
   })
   public void testImplicitEventHasCorrectAttributes()
   {
      Bean<Event<StudentRegisteredEvent>> event = getBeans(new TypeLiteral<Event<StudentRegisteredEvent>>() {}, new AnyLiteral()).iterator().next();
      assert event != null;
      assert event.getDeploymentType().equals(Standard.class);
      assert event.getScopeType().equals(Dependent.class);
      assert event.getName() == null;
      assert event.getBindings().size() == 1;
      assert event.getBindings().iterator().next().equals(new AnyLiteral());
      assert event.isSerializable();
   }
   
   @Test
   @SpecAssertion(section = "10.4.2", id = "f")
   public void testImplicitEventImplementationProvidedByContainer()
   {
      assert getInstanceByType(Registration.class).getCourseFullEvent() != null;

   }
}
