package org.jboss.jsr299.tck.tests.event.observer;

import java.lang.annotation.Annotation;

import javax.enterprise.event.Observer;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
public class ObserverTest extends AbstractJSR299Test
{
   public static class AnEventType
   {
   }

   public static class AnObserver implements Observer<AnEventType>
   {
      public boolean wasNotified = false;

      public boolean notify(AnEventType event)
      {
         wasNotified = true;
         return false;
      }
   }

   @Test(groups = { "events" })
   @SpecAssertions( {
      @SpecAssertion(section = "10.2", id = "h"),
      @SpecAssertion(section = "11.2.7", id = "b"),
      @SpecAssertion(section = "10.6", id = "a")
   })
   public void testObserverNotifiedWhenEventTypeAndAllBindingsMatch()
   {
      Annotation roleBinding = new RoleBinding("Admin");
      AnObserver observer1 = new AnObserver();
      AnObserver observer2 = new AnObserver();
      getCurrentManager().addObserver(observer1);
      getCurrentManager().addObserver(observer2, roleBinding);

      // Fire an event that will be delivered to the two above observers
      AnEventType anEvent = new AnEventType();
      getCurrentManager().fireEvent(anEvent, roleBinding);

      assert observer1.wasNotified;
      assert observer2.wasNotified;
      observer1.wasNotified = false;
      observer2.wasNotified = false;
      
      // Fire an event that will be delivered to only one
      getCurrentManager().fireEvent(anEvent);
      assert observer1.wasNotified;
      assert !observer2.wasNotified;
      observer1.wasNotified = false;
      observer2.wasNotified = false;
      
      // Also make sure the binding value is considered
      getCurrentManager().fireEvent(anEvent, new RoleBinding("user"));
      assert observer1.wasNotified;
      assert !observer2.wasNotified;
   }

}
