package org.jboss.jsr299.tck.tests.event.observer.abortProcessing;

import javax.enterprise.event.Observer;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
public class ObserverExceptionAbortsProcessingTest extends AbstractJSR299Test
{
   public static class AnEventType
   {
   }

   public static class AnObserverWithException implements Observer<AnEventType>
   {
      public boolean wasNotified = false;
      public RuntimeException theException = new RuntimeException("RE1");

      public boolean notify(AnEventType event)
      {
         wasNotified = true;
         throw theException;
      }
   }

   @Test(groups = { "events" })
   @SpecAssertion(section = "10.6", id = "c")
   public void testObserverThrowsExceptionAbortsNotifications()
   {
      AnObserverWithException observer = new AnObserverWithException();
      AnObserverWithException anotherObserver = new AnObserverWithException();
      getCurrentManager().addObserver(anotherObserver);
      getCurrentManager().addObserver(observer);

      // Fire an event that will be delivered to the two above observers
      AnEventType anEvent = new AnEventType();
      boolean fireFailed = false;
      try
      {
         getCurrentManager().fireEvent(anEvent);
      }
      catch (Exception e)
      {
         if (e.equals(observer.theException))
         {
            fireFailed = true;
            assert observer.wasNotified;
            assert !anotherObserver.wasNotified;
         }
         else if (e.equals(anotherObserver.theException))
         {
            fireFailed = true;
            assert !observer.wasNotified;
            assert anotherObserver.wasNotified;
         }
      }
      assert fireFailed;
   }
}
