/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.jsr299.tck.tests.event.observer.registerUsingEvent;

import java.util.Set;

import javax.enterprise.event.Observer;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * Tests proper functioning of the implicit event bean to register
 * observers and have those observers observe the events fired
 * from the same event bean or matching the same event type.
 * 
 * Spec version: 20090519
 * 
 * @author David Allen
 * @author Dan Allen
 */
@Artifact
@BeansXml("beans.xml")
public class RegisterObserversUsingImplicitEventTest extends AbstractJSR299Test
{
   @Test(groups = { "events" })
   @SpecAssertions( {
      @SpecAssertion(section = "10.4", id = "a"),
      @SpecAssertion(section = "10.4", id = "b"),
      @SpecAssertion(section = "10.4", id = "c"),
      @SpecAssertion(section = "10.4", id = "e"),
      @SpecAssertion(section = "10.4.1", id = "c"),
      @SpecAssertion(section = "10.4.1", id = "d"),
      @SpecAssertion(section = "11.2.7", id = "c")
   })
   public void testRegisterObserversObserveFiredEvents() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            Integer payload = new Integer(7);
            Set<Observer<Integer>> observers = getCurrentManager().resolveObservers(payload);
            assert observers.size() == 0;
            
            BlueFacedParrotFinch bean = getInstanceByType(BlueFacedParrotFinch.class);
            bean.methodThatRegistersIntegerObserver();

            observers = getCurrentManager().resolveObservers(payload);
            assert observers.size() == 1;
            //PLM - commenting out this line as it's now no longer possible to remove an observer
            // getCurrentManager().removeObserver(observers.iterator().next());
         }

      }.run();
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            BlueFacedParrotFinch.simpleStringEventObserved = false;
            BlueFacedParrotFinch bean = getInstanceByType(BlueFacedParrotFinch.class);
            bean.methodThatRegistersStringObserver();
            bean.methodThatFiresStringEvent();
            assert BlueFacedParrotFinch.simpleStringEventObserved;
         }

      }.run();
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            // When BirdCage is intantiated, the Mess produced by StarFinch is injected.
            // The producer fires an event of type Mess which the BirdKeeper observes.
            StarFinch starFinch = getInstanceByType(StarFinch.class);
            FinchKeeper birdKeeper = getInstanceByType(FinchKeeper.class);
            BirdCage birdCage = getInstanceByType(BirdCage.class);
            assert starFinch != null;
            assert birdCage != null;
            assert birdCage.getSomeMess() != null;
            assert birdKeeper.isNewMessDetected();
            birdKeeper.reset();
         }

      }.run();
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            // The initializer for OrangeCheekedWaxBill instantiates a Mess, assigns
            // it to a local property named someMess and fires an event using that object
            // with the binding type @Tame, which the BirdKeeper observes.
            OrangeCheekedWaxbill bird = getInstanceByType(OrangeCheekedWaxbill.class);
            FinchKeeper birdKeeper = getInstanceByType(FinchKeeper.class);
            assert bird != null;
            assert bird.getSomeMess() != null;
            assert birdKeeper.isNewMessDetected();
            birdKeeper.reset();
         }

      }.run();
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            // The initializer for AuroraFinch instantiates a Mess, assigns
            // it to a local property named someMess and fires an event using that object,
            // which the BirdKeeper observes.
            AuroraFinch bird = getInstanceByType(AuroraFinch.class);
            FinchKeeper birdKeeper = getInstanceByType(FinchKeeper.class);
            assert bird != null;
            assert bird.getSomeMess() != null;
            assert birdKeeper.isNewMessDetected();
            birdKeeper.reset();
         }

      }.run();
   }

}
