package org.jboss.jsr299.tck.tests.event.resolve.typeWithParameters;

import java.util.ArrayList;

import javax.enterprise.event.Observer;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
public class CheckTypeParametersWhenResolvingObserversTest extends AbstractJSR299Test
{
   public static class CharacterList extends ArrayList<Character>
   {
      private static final long serialVersionUID = 1L;
   }
   
   public static class StringList extends ArrayList<String>
   {
      private static final long serialVersionUID = 1L;
   }
   
   public static class IntegerList extends ArrayList<Integer>
   {
      private static final long serialVersionUID = 1L;
   }
   
   public static class StringListObserver implements Observer<ArrayList<String>>
   {
      public boolean wasNotified = false;

      public boolean notify(ArrayList<String> event)
      {
         wasNotified = true;
         return false;
      }
   }

   public static class IntegerListObserver implements Observer<ArrayList<Integer>>
   {
      public boolean wasNotified = false;

      public boolean notify(ArrayList<Integer> event)
      {
         wasNotified = true;
         return false;
      }
   }

   @Test(groups = { "events" })
   @SpecAssertions({
      @SpecAssertion(section = "10.5.1", id = "a"),
      @SpecAssertion(section = "11.2.9", id = "a")
   })
   public void testResolvingChecksTypeParameters()
   {
      StringListObserver observer = new StringListObserver();
      IntegerListObserver anotherObserver = new IntegerListObserver();
      getCurrentManager().addObserver(observer);
      getCurrentManager().addObserver(anotherObserver);
      assert getCurrentManager().resolveObservers(new StringList()).size() == 1;
      assert getCurrentManager().resolveObservers(new StringList()).iterator().next().getClass().equals(StringListObserver.class);
      assert getCurrentManager().resolveObservers(new CharacterList()).size() == 0;
   }
   
   @Test(groups = { "events" })
   @SpecAssertions({
      @SpecAssertion(section = "10.2", id = "h"),
      @SpecAssertion(section = "10.5", id = "a")
      // FIXME also 10.3.1, which does not yet have spec assertions cataloged
   })
   public void testResolvingChecksTypeParametersOnObservesMethod()
   {
      Foo<String> fooEvent = new Foo<String>() {};
      getCurrentManager().fireEvent(fooEvent);
      assert fooEvent.isObserved();
   }
}
