package org.jboss.jsr299.tck.tests.implementation.disposal.method.definition;

import java.lang.annotation.Annotation;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 */
@Artifact
@BeansXml("beans.xml")
public class DisposalMethodDefinitionTest extends AbstractJSR299Test
{
   private static final Annotation DEADLIEST_LITERAL = new AnnotationLiteral<Deadliest>() {};

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "2.3.5", id = "c"),
      @SpecAssertion(section = "3.4.5", id = "b"),
      @SpecAssertion(section = "3.4.5", id = "c"),
      @SpecAssertion(section = "3.4.5", id = "e"),
      @SpecAssertion(section = "3.4.6", id = "ba"),
      @SpecAssertion(section = "3.4.7", id = "a"),
      @SpecAssertion(section = "3.4.9", id = "aa")
   })
   public void testBindingTypesAppliedToDisposalMethodParameters() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Bean<Tarantula> tarantula = getBeans(Tarantula.class, DEADLIEST_LITERAL).iterator().next();
            CreationalContext<Tarantula> creationalContext = getCurrentManager().createCreationalContext(tarantula);
            Tarantula instance = tarantula.create(creationalContext);
            tarantula.destroy(instance, creationalContext);
         }
         
      }.run();
      assert SpiderProducer.isTameSpiderDestroyed();
      assert SpiderProducer.isCurrentSpiderDestroyed();
   }

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "3.4.5", id = "a"),
      @SpecAssertion(section = "3.4.6", id = "ba")
   })
   public void testDisposalMethodOnNonBean() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Bean<Tarantula> tarantula = getBeans(Tarantula.class, DEADLIEST_LITERAL).iterator().next();
            CreationalContext<Tarantula> creationalContext = getCurrentManager().createCreationalContext(tarantula);
            Tarantula instance = getCurrentManager().getContext(tarantula.getScopeType()).get(tarantula);
            tarantula.destroy(instance, creationalContext);
         }
         
      }.run();
      assert !DisposalNonBean.isSpiderDestroyed();
   }

   /**
    * In addition to the disposed parameter, a disposal method may declare
    * additional parameters, which may also specify bindings. The container
    * calls Manager.getInstanceToInject() to determine a value for each
    * parameter of a disposal method and calls the disposal method with those
    * parameter values
    * @throws Exception
    */
   @Test(groups = { "disposalMethod" })
   @SpecAssertion(section = "3.4.8", id = "a")
   public void testDisposalMethodParametersGetInjected() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Bean<Tarantula> tarantula = getBeans(Tarantula.class, DEADLIEST_LITERAL).iterator().next();
            CreationalContext<Tarantula> creationalContext = getCurrentManager().createCreationalContext(tarantula);
            Tarantula instance = getCurrentManager().getContext(tarantula.getScopeType()).get(tarantula);
            tarantula.destroy(instance, creationalContext);
         }
         
      }.run();
      assert SpiderProducer.isDeadliestSpiderDestroyed();
   }

}
