package org.jboss.jsr299.tck.tests.implementation.enterprise.definition;

import java.lang.annotation.Annotation;

import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * @author Nicklas Karlsson
 * 
 * Spec version: 20090519
 */
@Artifact
@Packaging(PackagingType.EAR)
@BeansXml("beans.xml")
public class EnterpriseBeanDefinitionTest extends AbstractJSR299Test
{
   @Test(groups = { "enterpriseBeans" })
   @SpecAssertion(section = "3.3", id = "b")
   public void testStatelessMustBeDependentScoped()
   {
      assert getBeans(GiraffeLocal.class).size() == 1;
      assert getBeans(GiraffeLocal.class).iterator().next().getScopeType().equals(Dependent.class);
   }

   @Test(groups = { "enterpriseBeans" })
   @SpecAssertion(section = "3.3", id = "c")
   public void testSingletonWithDependentScopeOK()
   {
      assert getBeans(Labrador.class).size() == 1;
   }

   @Test(groups = { "enterpriseBeans" })
   @SpecAssertion(section = "3.3", id = "c")
   public void testSingletonWithApplicationScopeOK()
   {
      assert getBeans(Laika.class).size() == 1;
   }

   @Test(groups = { "enterpriseBeans" })
   @SpecAssertions( {
      @SpecAssertion(section = "3.2.1", id = "f"),
      @SpecAssertion(section = "3.3.2", id = "aa"),
      @SpecAssertion(section = "3.3.3", id = "c") } )
   public void testBeanTypesAreLocalInterfacesWithoutWildcardTypesOrTypeVariablesWithSuperInterfaces()
   {
      Bean<DogLocal> dogBean = getBeans(DogLocal.class).iterator().next();
      assert dogBean.getTypes().contains(DogLocal.class);
      assert dogBean.getTypes().contains(PitbullLocal.class);
      assert !dogBean.getTypes().contains(Pitbull.class);
   }

   /**
    * If the EJB has a bean class local view and the bean class is not a
    * parameterized type, the set of bean types contains the bean class and all
    * superclasses.
    */
   @Test(groups = { "ejb 3.1", "ri-broken" })
   @SpecAssertion(section = "3.3.2", id = "ba")
   public void testEnterpriseBeanWithLocalViewAndNotParameterizedTypeHasClassAndSuperclassesAsBeanTypes()
   {
      //TODO We need a 3.1 compliant container for this test
      Bean<DogLocal> dogBean = getBeans(DogLocal.class).iterator().next();
      assert dogBean.getTypes().contains(Pitbull.class);
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertions({
      @SpecAssertion(section = "3.3.2", id = "c"),
      @SpecAssertion(section = "3.3.3", id = "a"),
      @SpecAssertion(section = "2.2", id = "l")
   })
   public void testObjectIsInAPITypes()
   {
      assert getBeans(GiraffeLocal.class).size() == 1;
      assert getBeans(GiraffeLocal.class).iterator().next().getTypes().contains(Object.class);
   }

   @Test(groups = { "enterpriseBeans" })
   @SpecAssertion(section = "3.3.2", id = "d")
   public void testRemoteInterfacesAreNotInAPITypes()
   {
      Bean<DogLocal> dogBean = getBeans(DogLocal.class).iterator().next();
      assert !dogBean.getTypes().contains(DogRemote.class);
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.3", id = "ba")
   public void testBeanWithScopeAnnotation()
   {
      Bean<LionLocal> lionBean = getBeans(LionLocal.class).iterator().next();
      assert lionBean.getScopeType().equals(RequestScoped.class);
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.3", id = "bb")
   public void testBeanWithNamedAnnotation()
   {
      Bean<MonkeyLocal> monkeyBean = getBeans(MonkeyLocal.class).iterator().next();
      assert monkeyBean.getName().equals("Monkey");
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.3", id = "bc")
   public void testBeanWithDeploymentType()
   {
      Bean<MonkeyLocal> monkeyBean = getBeans(MonkeyLocal.class).iterator().next();
      assert monkeyBean.getDeploymentType().equals(AnotherDeploymentType.class);
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.3", id = "bd")
   public void testBeanWithStereotype()
   {
      Bean<PolarBearLocal> polarBearBean = getBeans(PolarBearLocal.class).iterator().next();
      assert polarBearBean.getDeploymentType().equals(AnotherDeploymentType.class);
      assert polarBearBean.getScopeType().equals(RequestScoped.class);
      assert polarBearBean.getName().equals("polarBear");
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.3", id = "be")
   public void testBeanWithBindings()
   {
      Annotation tame = new AnnotationLiteral<Tame>(){};
      Bean<ApeLocal> apeBean = getBeans(ApeLocal.class, tame).iterator().next();
      assert apeBean.getBindings().contains(tame);
   }

   @Test(groups = "enterpriseBeans")
   @SpecAssertion(section = "3.3.5", id = "a")
   public void testDefaultName()
   {
      assert getBeans(PitbullLocal.class).size() == 1;
      assert getBeans(PitbullLocal.class).iterator().next().getName().equals("pitbull");
   }

}
