package org.jboss.jsr299.tck.tests.implementation.enterprise.newBean;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.NewLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 */
@Artifact
@Packaging(PackagingType.EAR)
@IntegrationTest
public class NewEnterpriseBeanICTest extends AbstractJSR299Test
{

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "m")
   public void testNewBeanHasSameInitializers()
   {
      InitializerSimpleBeanLocal bean = getInstanceByType(InitializerSimpleBeanLocal.class);
      InitializerSimpleBeanLocal newBean = getInstanceByType(InitializerSimpleBeanLocal.class,new NewLiteral());
      assert bean != newBean;
      assert bean.getInitializerCalls() == 2;
   }

   /**
    * Sets up both the bean and the @New bean with different configurations
    * so that the correct producer method used can be determined.
    * 
    * @throws Exception 
    */
   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "v")
   public void testNewBeanHasNoProducerMethods() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            FoxLocal fox = getInstanceByType(FoxLocal.class);
            FoxLocal newFox = getInstanceByType(FoxLocal.class,new NewLiteral());
            fox.setNextLitterSize(3);
            newFox.setNextLitterSize(5);
            Litter theOnlyLitter = getInstanceByType(Litter.class,new AnnotationLiteral<Tame>(){});
            assert theOnlyLitter.getQuantity() == fox.getNextLitterSize();
         }
         
      }.run();
   }

   /**
    * TODO Once WBRI-208 is fixed, check this test too
    * 
    * @throws Exception
    */
   @Test(groups = { "new", "ri-broken" })
   @SpecAssertion(section = "3.11", id = "w")
   public void testNewBeanHasNoProducerFields() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            FoxLocal fox = getInstanceByType(FoxLocal.class);
            FoxLocal newFox = getInstanceByType(FoxLocal.class,new NewLiteral());
            newFox.setDen(new Den("NewFoxDen"));
            Den theOnlyDen = getInstanceByType(Den.class,new AnnotationLiteral<Tame>(){});
            assert theOnlyDen.getName().equals(fox.getDen().getName());
         }
         
      }.run();
   }

   @Test(groups = { "new", "ri-broken", "disposal" })
   @SpecAssertion(section = "3.11", id = "x")
   public void testNewBeanHasNoDisposalMethods() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            FoxLocal fox = getInstanceByType(FoxLocal.class);
            FoxLocal newFox = getInstanceByType(FoxLocal.class,new NewLiteral());
            Bean<Litter> litterBean = getBeans(Litter.class).iterator().next();
            CreationalContext<Litter> creationalContext = getCurrentManager().createCreationalContext(litterBean);
            Litter litter = getInstanceByType(Litter.class);
            litterBean.destroy(litter, creationalContext);
            assert fox.isLitterDisposed();
            assert !newFox.isLitterDisposed();
         }
         
      }.run();
   }

}
