package org.jboss.jsr299.tck.tests.implementation.enterprise.newBean;

import java.lang.annotation.Annotation;
import java.util.Set;

import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.New;
import javax.enterprise.inject.deployment.Standard;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.AnyLiteral;
import org.jboss.jsr299.tck.literals.CurrentLiteral;
import org.jboss.jsr299.tck.literals.NewLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
@Packaging(PackagingType.EAR)
public class NewEnterpriseBeanTest extends AbstractJSR299Test
{
   private Bean<WrappedEnterpriseBeanLocal> newEnterpriseBean;
   
   private static final Annotation TAME_LITERAL = new AnnotationLiteral<Tame>() {};
   
   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "p")
   public void testNewBeanIsDependentScoped()
   {
      Set<Bean<WrappedEnterpriseBeanLocal>> beans = getBeans(WrappedEnterpriseBeanLocal.class, new NewLiteral());
      assert beans.size() == 1;
      newEnterpriseBean = beans.iterator().next();
      assert Dependent.class.equals(newEnterpriseBean.getScopeType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "q")
   public void testNewBeanIsOfStandardDeploymentType()
   {
      Set<Bean<WrappedEnterpriseBeanLocal>> beans = getBeans(WrappedEnterpriseBeanLocal.class, new NewLiteral());
      assert beans.size() == 1;
      newEnterpriseBean = beans.iterator().next();
      assert Standard.class.equals(newEnterpriseBean.getDeploymentType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "r")
   public void testNewBeanIsHasOnlyNewBinding()
   {
      Set<Bean<WrappedEnterpriseBeanLocal>> beans = getBeans(WrappedEnterpriseBeanLocal.class, new NewLiteral());
      assert beans.size() == 1;
      newEnterpriseBean = beans.iterator().next();
      assert newEnterpriseBean.getBindings().size() == 1;
      assert newEnterpriseBean.getBindings().iterator().next().annotationType().equals(new NewLiteral().annotationType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "s")
   public void testNewBeanHasNoWebBeanName()
   {
      Set<Bean<WrappedEnterpriseBeanLocal>> beans = getBeans(WrappedEnterpriseBeanLocal.class, new NewLiteral());
      assert beans.size() == 1;
      newEnterpriseBean = beans.iterator().next();
      assert newEnterpriseBean.getName() == null;
   }

   @Test(groups = { "new"})
   @SpecAssertion(section = "3.11", id = "t")
   public void testNewBeanHasNoStereotypes()
   {
      Bean<MonkeyLocal> monkeyBean = getBeans(MonkeyLocal.class).iterator().next();
      Bean<MonkeyLocal> newMonkeyBean = getBeans(MonkeyLocal.class, new NewLiteral()).iterator().next();
      assert monkeyBean.getScopeType().equals(RequestScoped.class);
      assert newMonkeyBean.getScopeType().equals(Dependent.class);
      assert monkeyBean.getName().equals("monkey");
      assert newMonkeyBean.getName() == null;
   }
   
   @Test(groups = {"new" })
   @SpecAssertion(section = "3.11", id = "u")
   public void testNewBeanHasNoObservers()
   {
      // Should just be 1 observer from bean, not new bean
      assert getCurrentManager().resolveObservers("event").size() == 1;
   }

   @Test
   @SpecAssertions({
      @SpecAssertion(section="3.11", id = "j")
   })
   public void testForEachEnterpriseBeanANewBeanExists()
   {
      Bean<OrderLocal> orderBean = getBeans(OrderLocal.class).iterator().next();
      Bean<OrderLocal> newOrderBean = getBeans(OrderLocal.class, new NewLiteral()).iterator().next();
      assert orderBean.getBindings().size() == 2;
      assert orderBean.getBindings().contains(new CurrentLiteral());
      assert orderBean.getBindings().contains(new AnyLiteral());
      
      assert getBeans(OrderLocal.class, new NewLiteral()).size() == 1;
      assert newOrderBean.getBindings().size() == 1;
      assert newOrderBean.getBindings().iterator().next().annotationType().equals(New.class);
      
      assert orderBean.getTypes().equals(newOrderBean.getTypes());
      
      Bean<LionLocal> lionBean = getBeans(LionLocal.class, TAME_LITERAL).iterator().next();
      Bean<LionLocal> newLionBean = getBeans(LionLocal.class, new NewLiteral()).iterator().next();
      assert getBeans(LionLocal.class, TAME_LITERAL).size() == 1;
      assert lionBean.getBindings().size() == 2;
      assert lionBean.getBindings().contains(TAME_LITERAL);
      assert lionBean.getBindings().contains(new AnyLiteral());
      
      assert getBeans(LionLocal.class, new NewLiteral()).size() == 1;
      assert newLionBean.getBindings().size() == 1;
      assert newLionBean.getBindings().iterator().next().annotationType().equals(New.class);
      
      assert lionBean.getTypes().equals(newLionBean.getTypes());
   }
}
