package org.jboss.jsr299.tck.tests.implementation.enterprise.remove;

import static org.jboss.testharness.impl.packaging.PackagingType.EAR;

import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 *
 * @author Nicklas Karlsson
 */
@Artifact
@Packaging(EAR)
@IntegrationTest
public class EnterpriseBeanRemoveMethodTest extends AbstractJSR299Test
{
   
   @Test(groups = { "enterpriseBeans", "removeMethod", "lifecycle" })
   @SpecAssertion(section = "3.3.1", id = "a")
   public void applicationMayCallAnyRemoveMethodOnDependentScopedSessionEnterpriseBeans() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            DependentSessionInterface sessionBean = getInstanceByType(DependentSessionInterface.class);
            sessionBean.remove();
            Bean<?> bean = getCurrentManager().getBeans(StateKeeper.class).iterator().next();
            StateKeeper stateKeeper = (StateKeeper)
               getCurrentManager().getReference(bean,StateKeeper.class, getCurrentManager().createCreationalContext(bean));
            assert stateKeeper.isRemoveCalled();
         }
      }.run();
   }

   @Test(groups = { "enterpriseBeans", "removeMethod", "lifecycle" })
   @SpecAssertion(section = "3.3.1", id = "da")
   public void applicationMayCallRemoveMethodOnDependentScopedSessionEnterpriseBeansButNoParametersArePassed() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            DependentSessionInterface sessionBean = getInstanceByType(DependentSessionInterface.class);
            sessionBean.anotherRemoveWithParameters("required", null);
            StateKeeper stateKeeper = getInstanceByType(StateKeeper.class);
            assert stateKeeper.isRemoveCalled();
         }
      }.run();
   }

   @Test(groups = { "enterpriseBeans", "removeMethod", "lifecycle" }, expectedExceptions = UnsupportedOperationException.class)
   @SpecAssertions({
      @SpecAssertion(section = "3.3.1", id = "b"),
      @SpecAssertion(section = "3.3.1", id = "c")
   })
   public void applicationCannotCallRemoveMethodOnNonDependentScopedSessionEnterpriseBean()
   {
      SessionScopedSessionInterface sessionBean = getInstanceByType(SessionScopedSessionInterface.class);
      sessionBean.remove();
      assert false : "Should never reach this assertion";
   }

}
