package org.jboss.jsr299.tck.tests.implementation.producer.field.lifecycle;

import javax.enterprise.inject.IllegalProductException;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
@BeansXml("beans.xml")
public class ProducerFieldLifecycleTest extends AbstractJSR299Test
{

   @Test(groups = { "producerField" })
   @SpecAssertions({
      @SpecAssertion(section = "7.6", id = "b")
   })
   public void testProducerFieldNotAnotherBean()
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getInstanceByType(BrownRecluse.class) != null;
         }
      };
   }

   @Test(groups = { "producerField" })
   @SpecAssertions({
      @SpecAssertion(section = "5.4.7", id = "a"),
      @SpecAssertion(section = "3.5", id = "b")
   })
   public void testProducerStaticFieldBean()
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            TarantulaConsumer tarantulaConsumer = getInstanceByType(TarantulaConsumer.class);
            assert tarantulaConsumer.getConsumedTarantula().equals(StaticTarantulaProducer.produceTarantula);
         }
      };
   }

   @Test(groups = { "producerField" })
   @SpecAssertions({
     @SpecAssertion(section = "4.3.2", id = "a"),
     @SpecAssertion(section = "5.4.7", id = "b"),
     @SpecAssertion(section = "7.6", id = "ga"),
     @SpecAssertion(section = "7.6", id = "a")
   })
   public void testProducerFieldBeanCreate() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            BlackWidowConsumer spiderConsumer = getInstanceByType(BlackWidowConsumer.class);
            assert spiderConsumer.getInjectedSpider().equals(BlackWidowProducer.blackWidow);
         }
      }.run();
   }

   @Test(groups = { "producerField", "specialization" })
   @SpecAssertions({
      @SpecAssertion(section = "4.3.2", id = "a")
   })
   public void testSpecializedBeanAlwaysUsed() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            TarantulaConsumer spiderConsumer = getInstanceByType(TarantulaConsumer.class);
            assert spiderConsumer.getConsumedTarantula() != null;
            assert spiderConsumer.getConsumedTarantula() instanceof DefangedTarantula;
         }
      }.run();
   }

   @Test(groups = { "producerField" })
   @SpecAssertions({
      @SpecAssertion(section = "3.5", id = "d"),
      @SpecAssertion(section = "7.6", id = "m")
    })    
   public void testProducerFieldReturnsNullIsDependent() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            NullSpiderConsumer consumerBean = getInstanceByType(NullSpiderConsumer.class);
            assert consumerBean.getInjectedSpider() == null;
         }
      }.run();
   }

   @Test(groups = { "producerField" }, expectedExceptions = IllegalProductException.class)  
   @SpecAssertions({
     @SpecAssertion(section = "3.5", id = "e"),
     @SpecAssertion(section = "7.6", id = "n")
   })   
   public void testProducerFieldReturnsNullIsNotDependent() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            NullSpiderConsumerForBrokenProducer consumer = getInstanceByType(NullSpiderConsumerForBrokenProducer.class);
            // The injected spider is proxied since it is in the request scope.
            // So to actually create the bean instance, we need to invoke
            // some method on the proxy.
            if (consumer.getInjectedSpider() == null)
            {
               // No proxy in this impl, and no exception = fail
            }
            else
            {
               // Proxy is assumed, so invoke some method
               consumer.getInjectedSpider().bite();
            }
         }
      }.run();
   }

}
