package org.jboss.jsr299.tck.tests.implementation.producer.method.lifecycle;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.IllegalProductException;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 * 
 */
@Artifact
@BeansXml("beans.xml")
public class ProducerMethodLifecycleTest extends AbstractJSR299Test
{

   @Test(groups = { "producerMethod", "disposalMethod" })
   @SpecAssertions({
      @SpecAssertion(section = "7", id = "f"),
      @SpecAssertion(section = "7.5", id = "ma")
   })
   public void testProducerMethodBeanDestroy() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            SpiderProducer.setTarantulaDestroyed(false);
            Tarantula tarantula = getInstanceByType(Tarantula.class);
            Bean<Tarantula> tarantulaBean = getBeans(Tarantula.class).iterator().next();
            CreationalContext<Tarantula> creationalContext = getCurrentManager().createCreationalContext(tarantulaBean);
            tarantulaBean.destroy(tarantula, creationalContext);
            assert SpiderProducer.isTarantulaDestroyed();
            assert SpiderProducer.isDestroyArgumentsSet();
            assert !SpiderProducerNotUsed.isTarantulaDestroyed();
         }
         
      }.run();
   }

   @Test(groups = { "producerMethod" })
   @SpecAssertions({
      @SpecAssertion(section = "7", id = "b")
   })
   public void testProducerMethodBeanCreate() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(Tarantula.class);
            assert SpiderProducer.isTarantulaCreated();
         }
         
      }.run();
   }

   @Test(groups = { "producerMethod" })
   @SpecAssertions({
      @SpecAssertion(section = "7.5", id = "ea")
   })
   public void testProducerMethodInvokedOnCreate() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Bean<SpiderEgg> eggBean = getBeans(SpiderEgg.class).iterator().next();
            CreationalContext<SpiderEgg> creationalContext = getCurrentManager().createCreationalContext(eggBean);
            assert eggBean.create(creationalContext) != null;
         }
         
      }.run();      
   }

   @Test(groups = { "producerMethod", "rewrite" })
   //Test needs to user the InjectionTarget interface for these assertions
   @SpecAssertions({
      @SpecAssertion(section = "11.4.1", id = "g")
   })
   public void testProducerMethodFromMostSpecializedBeanUsed() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            Tarantula.setNumberCreated(0);
            Bean<Tarantula> spiderBean = getBeans(Tarantula.class).iterator().next();
            CreationalContext<Tarantula> creationalContext = getCurrentManager().createCreationalContext(spiderBean);            
            Tarantula tarantula = spiderBean.create(creationalContext);
            assert tarantula.getValue().equals("Pete");
            assert Tarantula.getNumberCreated() == 1;
         }
      }.run();      
   }
   
   @Test(groups = { "producerMethod" })
   @SpecAssertions({
      @SpecAssertion(section = "7.5", id = "k")
   })
   public void testCreateReturnsNullIfProducerDoesAndDependent() throws Exception
   {
      Bean<Bread> breadBean = getBeans(Bread.class).iterator().next();
      CreationalContext<Bread> creationalContext = getCurrentManager().createCreationalContext(breadBean);       
      assert breadBean.create(creationalContext) == null;
   }

   @Test(groups = { "producerMethod", "broken" }, expectedExceptions = IllegalProductException.class )
   @SpecAssertions({
      @SpecAssertion(section = "7.5", id = "l")
   })
   // TODO I'm not quite sure what this test is currently doing!
   public void testCreateFailsIfProducerReturnsNullAndNotDependent() throws Exception
   {
      Bean<PotatoChip> chipBean = getBeans(PotatoChip.class).iterator().next();
      CreationalContext<PotatoChip> creationalContext = getCurrentManager().createCreationalContext(chipBean);       
      assert chipBean.create(creationalContext) == null;
   }
}
