package org.jboss.jsr299.tck.tests.implementation.simple.definition;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.tests.implementation.simple.definition.OuterClass.InnerClass_NotBeean;
import org.jboss.jsr299.tck.tests.implementation.simple.definition.OuterClass.StaticInnerClass;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 * 
 */
@Artifact
public class SimpleBeanDefinitionTest extends AbstractJSR299Test
{

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "3.2.1", id = "ca"),
      @SpecAssertion(section = "3.2", id = "ca")
   })
   public void testAbstractClassDeclaredInJavaNotDiscovered()
   {
      assert getBeans(Cow_NotBean.class).size() == 0;
   }

   @Test(groups = "innerClass")
   @SpecAssertions({
      @SpecAssertion(section = "3.2.1", id = "ba")
   })
   public void testStaticInnerClassDeclaredInJavaAllowed()
   {
      assert getBeans(StaticInnerClass.class).size() == 1;
   }

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "3.2.1", id = "b")
   })
   public void testNonStaticInnerClassDeclaredInJavaNotDiscovered()
   {
      assert getBeans(InnerClass_NotBeean.class).size() == 0;
   }

   @Test
   @SpecAssertion(section = "3.2.1", id = "cb")
   public void testInterfaceNotDiscoveredAsSimpleBean()
   {
      assert getBeans(Car.class).size() == 0;
   }

   //Not sure yet whether this is needed
//   @Test
//   public void testClassesImplementingServletInterfacesNotDiscoveredAsSimpleBeans()
//   {
//      assert getBeans(MockFilter.class).size() == 0;
//      assert getBeans(MockHttpSessionListener.class).size() == 0;
//      assert getBeans(MockServlet.class).size() == 0;
//      assert getBeans(MockServletContextListener.class).size() == 0;
//      assert getBeans(MockServletRequestListener.class).size() == 0;
//   }

   @Test
   @SpecAssertion(section="3.2.1", id="f")
   public void testClassesImplementingEnterpriseBeanInterfaceNotDiscoveredAsSimpleBean()
   {
      assert getBeans(MockEnterpriseBean.class).size() == 0;
   }

   @Test
   @SpecAssertion(section="3.2.1", id="p")
   public void testSimpleBeanOnlyIfConstructorParameterless()
   {
      assert getBeans(Antelope_NotBean.class).isEmpty();
      assert !getBeans(Donkey.class).isEmpty();
   }

   @Test
   @SpecAssertion(section="3.2.1", id="q")
   public void testSimpleBeanOnlyIfConstructorIsInitializer()
   {
      assert getBeans(Antelope_NotBean.class).isEmpty();
      assert !getBeans(Sheep.class).isEmpty();
   }

   @Test
   @SpecAssertion(section = "3.2.4.1", id = "a")
   public void testInitializerAnnotatedConstructor() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            Sheep.constructedCorrectly = false;
            getInstanceByType(Sheep.class);
            assert Sheep.constructedCorrectly;

         }
      }.run();

   }

   @Test
   @SpecAssertions({
      @SpecAssertion(section = "3.2.5", id = "a"), 
      @SpecAssertion(section = "3.2.4.1", id = "ba"),
      @SpecAssertion(section = "3.1", id = "a"),
      @SpecAssertion(section = "3.2.3", id = "a")
   })
   public void testEmptyConstructorUsed()
   {
      Donkey.constructedCorrectly = false;
      getInstanceByType(Donkey.class);
      assert Donkey.constructedCorrectly;
   }

   @Test
   @SpecAssertion(section = "3.2.4.1", id = "a")
   public void testInitializerAnnotatedConstructorUsedOverEmptyConstuctor() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(Turkey.class);
            assert Turkey.constructedCorrectly;
         }
      }.run();
   }

   @Test
   @SpecAssertion(section = "3.2", id = "fa")
   public void testDependentScopedBeanCanHavePublicField() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getInstanceByType(Tiger.class).name.equals("pete");
         }

      }.run();

   }

}
