package org.jboss.jsr299.tck.tests.implementation.simple.lifecycle;

import java.lang.annotation.Annotation;

import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.context.spi.Context;
import javax.enterprise.context.spi.Contextual;
import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.CreationException;
import javax.enterprise.inject.spi.Bean;

import org.apache.commons.httpclient.methods.GetMethod;
import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.impl.MockCreationalContext;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 * 
 */
@Artifact
@BeansXml("beans.xml")
public class SimpleBeanLifecycleTest extends AbstractJSR299Test
{
   private static final Annotation TAME_LITERAL = new AnnotationLiteral<Tame>()
   {
   };

   @Test(groups = "beanConstruction")
   @SpecAssertions({
     @SpecAssertion(section = "3.2.4.2", id = "b"),
     @SpecAssertion(section = "3.2.4.2", id = "c"),
     @SpecAssertion(section = "2.3.5", id = "d")
   })
   public void testInjectionOfParametersIntoBeanConstructor()
   {
      assert getBeans(FishPond.class).size() == 1;
      FishPond fishPond = getInstanceByType(FishPond.class);
      assert fishPond.goldfish != null;
      assert fishPond.goose != null;
   }
   
   @Test
   @SpecAssertion(section = "3.2.4.2", id = "b")
   public void testBindingTypeAnnotatedConstructor() throws Exception
   {
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(Duck.class);
            assert Duck.constructedCorrectly;
         }
      }.run();
   }

   @Test(groups = { "specialization" })
   @SpecAssertion(section = "3.2.5", id = "aa")
   public void testSpecializedBeanAlwaysUsed() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getInstanceByType(Lion.class,TAME_LITERAL) instanceof MountainLion;
         }

      }.run();
   }

   @Test(groups = "beanLifecycle")
   @SpecAssertions({
      @SpecAssertion(section = "6.1.1", id = "c"),
      @SpecAssertion(section = "6.1.1", id = "d"),
      @SpecAssertion(section = "6.1.1", id = "e")
   })
   public void testCreateReturnsSameBeanPushed() throws Exception
   {
      final CreationalContext<ShoeFactory> creationalContext = new MockCreationalContext<ShoeFactory>();
      final Contextual<ShoeFactory> bean = getBeans(ShoeFactory.class).iterator().next();
      MockCreationalContext.reset();
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            ShoeFactory instance = getCurrentManager().getContext(Dependent.class).get(bean, creationalContext);
            if (MockCreationalContext.isPushCalled())
            {
               assert instance.equals(MockCreationalContext.getLastBeanPushed());
            }
         }

      }.run();      
   }
      
   @Test(groups = "beanLifecycle")
   @SpecAssertions({
      @SpecAssertion(section = "6.1.1", id = "a"),
      @SpecAssertion(section="2", id="g"),
      @SpecAssertion(section="2.2.1", id="b"),
      @SpecAssertion(section="3.2.6", id="a"),
      @SpecAssertion(section = "7", id = "d")
   })
   public void testCreateReturnsInstanceOfBean()
   {
      assert getBeans(RedSnapper.class).size() == 1;
      assert getInstanceByType(RedSnapper.class) instanceof RedSnapper;
   }

   @Test(groups = "injection")
   @SpecAssertions({
      @SpecAssertion(section = "7.2", id = "c"),
      @SpecAssertion(section = "3.8.1", id = "a")
   })
   public void testCreateInjectsFieldsDeclaredInJava()
   {
      assert getBeans(TunaFarm.class).size() == 1;
      TunaFarm tunaFarm = getInstanceByType(TunaFarm.class);
      assert tunaFarm.tuna != null;
   }

   @Test(groups = "beanLifecycle")
   @SpecAssertions({
      @SpecAssertion(section = "6.2", id = "l")
   })
   public void testContextCreatesNewInstanceForInjection()
   {
      Context requestContext = getCurrentManager().getContext(RequestScoped.class);
      Bean<Tuna> tunaBean = getBeans(Tuna.class).iterator().next();
      assert requestContext.get(tunaBean) == null;
      TunaFarm tunaFarm = getInstanceByType(TunaFarm.class);
      assert tunaFarm.tuna != null;
   }

   @Test(groups = { "beanLifecycle", "lifecycleCallbacks" })
   @SpecAssertions({
     @SpecAssertion(section = "7.2", id = "d"),
     @SpecAssertion(section = "7.2", id = "e")
   })
   public void testPostConstructPreDestroy() throws Exception
   {
      assert getBeans(Farm.class).size() == 1;
      Bean<Farm> farmBean = getBeans(Farm.class).iterator().next();
      CreationalContext<Farm> creationalContext = getCurrentManager().createCreationalContext(farmBean);
      Farm farm = getInstanceByType(Farm.class);
      assert farm.founded != null;
      assert farm.initialStaff == 20;
      assert farm.closed == null;
      farmBean.destroy(farm, creationalContext);
      assert farm.closed != null;
   }

   @Test(groups = { "beanLifecycle", "lifecycleCallbacks" })
   @SpecAssertions({
     @SpecAssertion(section = "6.1.2", id = "a")
   })
   public void testContextualDestroyDisposesWhenNecessary() throws Exception
   {
      final Bean<Goose> gooseBean = getBeans(Goose.class).iterator().next();
      CreationalContext<Goose> creationalContext = getCurrentManager().createCreationalContext(gooseBean);
      final Goose goose = gooseBean.create(creationalContext);
      
      assert !EggProducer.isEggDisposed();
      assert !Egg.isEggDestroyed();
      gooseBean.destroy(goose, creationalContext);
      assert EggProducer.isEggDisposed();
   }

   @Test(groups = "beanLifecycle")
   @SpecAssertions({
      @SpecAssertion(section = "6.1.2", id = "a")
   })
   public void testContextualDestroyCatchesException()
   {
      Bean<Cod> codBean = getBeans(Cod.class).iterator().next();
      CreationalContext<Cod> creationalContext = getCurrentManager().createCreationalContext(codBean);      
      Cod codInstance = getInstanceByType(Cod.class);
      codInstance.ping();
      codBean.destroy(codInstance, creationalContext);
   }

   @Test(groups = "beanLifecycle")
   @SpecAssertions({
      @SpecAssertion(section = "6.1.2", id = "a"),
      @SpecAssertion(section = "6.4.3", id = "a")
   })
   public void testDependentsDestroyedAfterPreDestroy()
   {
      Bean<FishPond> pondBean = getBeans(FishPond.class).iterator().next();
      CreationalContext<FishPond> creationalContext = getCurrentManager().createCreationalContext(pondBean);
      FishPond fishPond = pondBean.create(creationalContext);
      pondBean.destroy(fishPond, creationalContext);
      assert Salmon.isBeanDestroyed();
   }

   @Test
   @SpecAssertion(section = "4.2", id = "baa")
   public void testSubClassInheritsPostConstructOnSuperclass() throws Exception
   {
      OrderProcessor.postConstructCalled = false;
      assert getBeans(CdOrderProcessor.class).size() == 1;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(CdOrderProcessor.class).order();
         }
      }.run();
      assert OrderProcessor.postConstructCalled;
   }
   
   @Test
   @SpecAssertion(section = "4.2", id = "bac")
   public void testIndirectSubClassInheritsPostConstructOnSuperclass() throws Exception
   {
      OrderProcessor.postConstructCalled = false;
      assert getBeans(IndirectOrderProcessor.class).size() == 1;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(IndirectOrderProcessor.class).order();
         }
      }.run();
      assert OrderProcessor.postConstructCalled;
   }   

   @Test
   @SpecAssertion(section = "4.2", id = "bba")
   public void testSubClassInheritsPreDestroyOnSuperclass() throws Exception
   {
      OrderProcessor.preDestroyCalled = false;
      assert getBeans(CdOrderProcessor.class).size() == 1;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {  
            Bean<CdOrderProcessor> bean = getBeans(CdOrderProcessor.class).iterator().next();
            CreationalContext<CdOrderProcessor> creationalContext = getCurrentManager().createCreationalContext(bean);      
            CdOrderProcessor instance = getInstanceByType(CdOrderProcessor.class);
            bean.destroy(instance, creationalContext);
         }
      }.run();
      assert OrderProcessor.preDestroyCalled;
   }
   
   @Test
   @SpecAssertion(section = "4.2", id = "bbc")
   public void testIndirectSubClassInheritsPreDestroyOnSuperclass() throws Exception
   {
      OrderProcessor.preDestroyCalled = false;
      assert getBeans(IndirectOrderProcessor.class).size() == 1;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {  
            Bean<IndirectOrderProcessor> bean = getBeans(IndirectOrderProcessor.class).iterator().next();
            CreationalContext<IndirectOrderProcessor> creationalContext = getCurrentManager().createCreationalContext(bean); 
            IndirectOrderProcessor instance = getInstanceByType(IndirectOrderProcessor.class);
            bean.destroy(instance, creationalContext);
         }
      }.run();
      assert OrderProcessor.preDestroyCalled;
   }   

   @Test
   @SpecAssertion(section = "4.2", id = "baa")
   public void testSubClassDoesNotInheritPostConstructOnSuperclassBlockedByIntermediateClass() throws Exception
   {
      assert getBeans(NovelOrderProcessor.class).size() == 1;
      OrderProcessor.postConstructCalled = false;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(NovelOrderProcessor.class).order();
         }
      }.run();
      assert !OrderProcessor.postConstructCalled;
   }

   @Test
   @SpecAssertion(section = "4.2", id = "bba")
   public void testSubClassDoesNotInheritPreDestroyConstructOnSuperclassBlockedByIntermediateClass() throws Exception
   {
      OrderProcessor.preDestroyCalled = false;
      assert getBeans(NovelOrderProcessor.class).size() == 1;
      new RunInDependentContext()
      {
         @Override
         protected void execute() throws Exception
         {
            Bean<NovelOrderProcessor> bean = getBeans(NovelOrderProcessor.class).iterator().next();
            CreationalContext<NovelOrderProcessor> creationalContext = getCurrentManager().createCreationalContext(bean);             
            NovelOrderProcessor instance = getInstanceByType(NovelOrderProcessor.class);
            bean.destroy(instance, creationalContext);
         }
      }.run();
      assert !OrderProcessor.preDestroyCalled;

   }

   @Test(expectedExceptions = CreationException.class)
   @SpecAssertion(section = "6.1.1", id = "b")
   public void testCreationExceptionWrapsCheckedExceptionThrownFromCreate() throws Exception
   {
      assert getBeans(Lorry_Broken.class).size() == 1;
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(Lorry_Broken.class);
         }

      }.run();
   }

   @Test(expectedExceptions = FooException.class)
   @SpecAssertion(section = "6.1.1", id = "b")
   public void testUncheckedExceptionThrownFromCreateNotWrapped() throws Exception
   {
      assert getBeans(Van_Broken.class).size() == 1;
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            getInstanceByType(Van_Broken.class);
         }

      }.run();
   }

}
