package org.jboss.jsr299.tck.tests.implementation.simple.newSimpleBean;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;
import java.util.Set;

import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.New;
import javax.enterprise.inject.deployment.Standard;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.AnyLiteral;
import org.jboss.jsr299.tck.literals.CurrentLiteral;
import org.jboss.jsr299.tck.literals.NewLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 * 
 */
@Artifact
public class NewSimpleBeanTest extends AbstractJSR299Test
{

   private static final Annotation TAME_LITERAL = new AnnotationLiteral<Tame>()
   {
   };

   @Test(groups = { "new" })
   @SpecAssertions({
      @SpecAssertion(section = "3.11", id = "p")
   })
   public void testNewBeanIsDependentScoped()
   {
      Set<Bean<ExplicitContructorSimpleBean>> beans = getBeans(ExplicitContructorSimpleBean.class, new NewLiteral());
      assert beans.size() == 1;
      Bean<ExplicitContructorSimpleBean> newSimpleBean = beans.iterator().next();
      assert Dependent.class.equals(newSimpleBean.getScopeType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "q")
   public void testNewBeanIsOfStandardDeploymentType()
   {
      Set<Bean<ExplicitContructorSimpleBean>> beans = getBeans(ExplicitContructorSimpleBean.class, new NewLiteral());
      assert beans.size() == 1;
      Bean<ExplicitContructorSimpleBean> newSimpleBean = beans.iterator().next();
      assert Standard.class.equals(newSimpleBean.getDeploymentType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "r")
   public void testNewBeanHasOnlyNewBinding()
   {
      Set<Bean<ExplicitContructorSimpleBean>> beans = getBeans(ExplicitContructorSimpleBean.class, new NewLiteral());
      assert beans.size() == 1;
      Bean<ExplicitContructorSimpleBean> newSimpleBean = beans.iterator().next();
      assert newSimpleBean.getBindings().size() == 1;
      assert newSimpleBean.getBindings().iterator().next().annotationType().equals(new NewLiteral().annotationType());
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "s")
   public void testNewBeanHasNoWebBeanName()
   {
      Set<Bean<ExplicitContructorSimpleBean>> beans = getBeans(ExplicitContructorSimpleBean.class, new NewLiteral());
      assert beans.size() == 1;
      Bean<ExplicitContructorSimpleBean> newSimpleBean = beans.iterator().next();
      assert newSimpleBean.getName() == null;
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "t")
   public void testNewBeanHasNoStereotypes()
   {
      Bean<Fox> foxBean = getBeans(Fox.class).iterator().next();
      assert foxBean.getScopeType().equals(RequestScoped.class);
      assert foxBean.getName().equals("fox");
      Bean<Fox> newFoxBean = getBeans(Fox.class, new NewLiteral()).iterator().next();
      assert newFoxBean.getScopeType().equals(Dependent.class);
      assert newFoxBean.getName() == null;
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "u")
   public void testNewBeanHasNoObservers()
   {
      // As long as only one observer exists here, we know it is not from the @New bean
      assert getCurrentManager().resolveObservers("event").size() == 1;
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "w")
   public void testNewBeanHasNoProducerFields() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Fox fox = getInstanceByType(Fox.class);
            Fox newFox = getInstanceByType(Fox.class,new NewLiteral());
            newFox.setDen(new Den("NewFoxDen"));
            Den theOnlyDen = getInstanceByType(Den.class);
            assert theOnlyDen.getName().equals(fox.getDen().getName());
         }
         
      }.run();
      
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "v")
   public void testNewBeanHasNoProducerMethods() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Fox fox = getInstanceByType(Fox.class);
            Fox newFox = getInstanceByType(Fox.class,new NewLiteral());
            fox.setNextLitterSize(3);
            newFox.setNextLitterSize(5);
            Litter theOnlyLitter = getInstanceByType(Litter.class);
            assert theOnlyLitter.getQuantity() == fox.getNextLitterSize();
         }
         
      }.run();
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "x")
   public void testNewBeanHasNoDisposalMethods() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Fox fox = getInstanceByType(Fox.class);
            Fox newFox = getInstanceByType(Fox.class,new NewLiteral());
            Bean<Litter> litterBean = getBeans(Litter.class).iterator().next();
            CreationalContext<Litter> creationalContext = getCurrentManager().createCreationalContext(litterBean);
            Litter litter = getInstanceByType(Litter.class);
            litterBean.destroy(litter, creationalContext);
            assert fox.isLitterDisposed();
            assert !newFox.isLitterDisposed();
         }
         
      }.run();
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "review", id = "review")
   public void testNewAnnotationCannotBeExplicitlyDeclared()
   {
      // All we can really do is make sure the annotation itself
      // prevents the compiler from violating the assertion
      Target target = New.class.getAnnotation(Target.class);
      for (ElementType elementType : target.value())
      {
         assert !elementType.equals(ElementType.TYPE);
      }
   }

   @Test
   @SpecAssertion(section = "3.11", id = "d")
   public void testForEachSimpleBeanANewBeanExists()
   {
      assert getCurrentManager().getBeans(Order.class).size() == 1;
      assert getCurrentManager().getBeans(Order.class).iterator().next().getBindings().size() == 2;
      assert getCurrentManager().getBeans(Order.class).iterator().next().getBindings().contains(new CurrentLiteral());

      assert getBeans(Order.class, new NewLiteral()).size() == 1;
      assert getBeans(Order.class, new NewLiteral()).iterator().next().getBindings().size() == 1;
      assert getBeans(Order.class, new NewLiteral()).iterator().next().getBindings().iterator().next().annotationType().equals(New.class);

      assert getCurrentManager().getBeans(Lion.class, TAME_LITERAL).size() == 1;
      assert getCurrentManager().getBeans(Lion.class, TAME_LITERAL).iterator().next().getBindings().size() == 2;
      assert getCurrentManager().getBeans(Lion.class, TAME_LITERAL).iterator().next().getBindings().contains(TAME_LITERAL);
      assert getCurrentManager().getBeans(Lion.class, TAME_LITERAL).iterator().next().getBindings().contains(new AnyLiteral());

      assert getBeans(Lion.class, new NewLiteral()).size() == 1;
      assert getBeans(Lion.class, new NewLiteral()).iterator().next().getBindings().size() == 1;
      assert getBeans(Lion.class, new NewLiteral()).iterator().next().getBindings().iterator().next().annotationType().equals(New.class);
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "f")
   public void testNewBeanHasSameConstructor()
   {
      ExplicitContructorSimpleBean.setConstructorCalls(0);
      ExplicitContructorSimpleBean bean = getInstanceByType(ExplicitContructorSimpleBean.class);
      ExplicitContructorSimpleBean newBean = getInstanceByType(ExplicitContructorSimpleBean.class,new NewLiteral());
      assert bean != newBean;
      assert ExplicitContructorSimpleBean.getConstructorCalls() == 2;
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "g")
   public void testNewBeanHasSameInitializers()
   {
      InitializerSimpleBean.setInitializerCalls(0);
      InitializerSimpleBean bean = getInstanceByType(InitializerSimpleBean.class);
      bean.businessMethod();  // Cause proxy to initialize the bean
      InitializerSimpleBean newBean = getInstanceByType(InitializerSimpleBean.class,new NewLiteral());
      assert bean != newBean;
      assert InitializerSimpleBean.getInitializerCalls() == 2;
   }

   @Test(groups = { "new" })
   @SpecAssertion(section = "3.11", id = "h")
   public void testNewBeanHasSameInjectedFields()
   {
      Bean<InitializerSimpleBean> simpleBean = getBeans(InitializerSimpleBean.class).iterator().next();
      Bean<InitializerSimpleBean> newSimpleBean = getBeans(InitializerSimpleBean.class, new NewLiteral()).iterator().next();
      assert !newSimpleBean.getInjectionPoints().isEmpty();
      assert simpleBean.getInjectionPoints().equals(newSimpleBean.getInjectionPoints());
   }
}
