/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.jsr299.tck.tests.implementation.simple.resource.ejb;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.jboss.testharness.impl.packaging.Resource;
import org.jboss.testharness.impl.packaging.Resources;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * EJB injection tests for simple beans.
 * 
 * Spec version: 20090519
 * 
 * @author David Allen
 *
 */
@Artifact
@Packaging(PackagingType.EAR)
@IntegrationTest
@Resources({
   @Resource(source="web.xml", destination="WEB-INF/web.xml")
})
@BeansXml("beans.xml")
public class EjbInjectionTest extends AbstractJSR299Test
{
   @Test(groups = { "ri-broken", "beanLifecycle", "commonAnnotations", "integration" })
   @SpecAssertion(section = "3.6.1", id = "ee")
   public void testInjectionOfEjbs()
   {
      Bean<SimpleBean> simpleBeanBean = getBeans(SimpleBean.class).iterator().next();
      CreationalContext<SimpleBean> creationalContext = getCurrentManager().createCreationalContext(simpleBeanBean);
      SimpleBean simpleBean = simpleBeanBean.create(creationalContext);
      assert !simpleBean.isMyEjbExists() : "Ejb was injected too early";
      assert simpleBean.getMyEjb() != null : "Ejb was not injected into bean";
      assert simpleBean.getMyEjb().hello().equals("hi!");
   }
}
