package org.jboss.jsr299.tck.tests.implementation.simple.resource.persistenceContext;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.IntegrationTest;
import org.jboss.testharness.impl.packaging.Packaging;
import org.jboss.testharness.impl.packaging.PackagingType;
import org.jboss.testharness.impl.packaging.Resource;
import org.jboss.testharness.impl.packaging.Resources;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * Injection of persistence related objects.
 * 
 * Spec version: 20090519
 *
 * @author David Allen
 *
 */
@Artifact
@Packaging(PackagingType.WAR)
@IntegrationTest
@Resources({
   @Resource(source="persistence.xml", destination="WEB-INF/persistence.xml")
})
@BeansXml("beans.xml")
public class PersistenceContextInjectionTest extends AbstractJSR299Test
{
   @Test(groups = { "beanLifecycle", "commonAnnotations", "integration" })
   @SpecAssertions( { 
      @SpecAssertion(section = "3.6.1", id = "cc")
   })
   public void testInjectionOfPersistenceContext()
   {
      Bean<SimpleBean> simpleBeanBean = getBeans(SimpleBean.class).iterator().next();
      CreationalContext<SimpleBean> creationalContext = getCurrentManager().createCreationalContext(simpleBeanBean);
      SimpleBean simpleBean = simpleBeanBean.create(creationalContext);
      EntityManager entityManager = simpleBean.getPersistenceContext();
      assert !simpleBean.isPersistenceContextExistsDuringConstruction() : "Persistence context injected too early";
      assert entityManager != null : "Persistence context was not injected into bean";
      assert entityManager.isOpen() : "persistence context not open injected into bean";
   }
   
   @Test(groups = { "broken", "beanLifecycle", "commonAnnotations", "integration" })
   @SpecAssertions( { 
      @SpecAssertion(section = "3.6.1", id = "dd")
   })
   public void testInjectionOfPersistenceUnit()
   {
      Bean<SimpleBean> simpleBeanBean = getBeans(SimpleBean.class).iterator().next();
      CreationalContext<SimpleBean> creationalContext = getCurrentManager().createCreationalContext(simpleBeanBean);
      SimpleBean simpleBean = simpleBeanBean.create(creationalContext);
      EntityManagerFactory entityManagerFactory = simpleBean.getPersistenceUnit();
      assert !simpleBean.isPersistenceUnitExistsDuringConstruction() : "Persistence unit injected too early";
      assert entityManagerFactory != null : "Persistence unit was not injected into bean";
      assert entityManagerFactory.isOpen() : "persistence unit not open injected into bean";
   }
}
