package org.jboss.jsr299.tck.tests.inheritance.specialization.simple;

import java.lang.annotation.Annotation;

import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.Any;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * Spec version: 20090519
 * 
 */
@Artifact
@BeansXml("beans.xml")
public class SimpleBeanSpecializationTest extends AbstractJSR299Test
{

   private static Annotation LANDOWNER_LITERAL = new AnnotationLiteral<Landowner>()
   {
   };

   @SuppressWarnings("unchecked")
   @Test
   @SpecAssertions( { 
      @SpecAssertion(section = "4.3.1", id = "i"), 
      @SpecAssertion(section = "4.3.1", id = "j"), 
      @SpecAssertion(section = "3.2.5", id = "aa") 
   })
   public void testSpecializingBeanHasBindingsOfSpecializedAndSpecializingBean()
   {
      assert getBeans(Farmer.class, LANDOWNER_LITERAL).size() == 1;
      Bean<?> bean = getBeans(Farmer.class, LANDOWNER_LITERAL).iterator().next();
      assert bean.getTypes().contains(LazyFarmer.class);
      assert bean.getBindings().size() == 3;
      assert annotationSetMatches(bean.getBindings(), Landowner.class, Lazy.class, Any.class);
   }

   @Test
   @SpecAssertions( { 
      @SpecAssertion(section = "4.3.1", id = "k"), 
      @SpecAssertion(section = "3.2.5", id = "aa") 
   })
   public void testSpecializingBeanHasNameOfSpecializedBean()
   {
      assert getBeans(Farmer.class, LANDOWNER_LITERAL).size() == 1;
      assert getBeans(Farmer.class, LANDOWNER_LITERAL).iterator().next().getTypes().contains(LazyFarmer.class);
      assert "farmer".equals(getBeans(Farmer.class, LANDOWNER_LITERAL).iterator().next().getName());
   }

   @Test
   public void testSpecializedBeanNotInstantiated() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Farmer farmer = getInstanceByType(Farmer.class,LANDOWNER_LITERAL);
            assert farmer.getClassName().equals(LazyFarmer.class.getName());
         }

      }.run();
   }

   @Test
   @SpecAssertions( { 
      @SpecAssertion(section = "4.3.2", id = "a")
   })
   public void testProducerMethodOnSpecializedBeanCalledOnSpecializingBean() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            assert getBeans(Waste.class).size() == 1;
            assert getInstanceByType(Waste.class).getFrom().equals(Office.class.getName());
         }

      }.run();
   }
}
