package org.jboss.jsr299.tck.tests.lookup.clientProxy;

import java.io.IOException;

import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
public class ClientProxyTest extends AbstractJSR299Test
{

   @Test
   @SpecAssertion(section = "5.3", id = "b")
   public void testClientProxyUsedForNormalScope()
   {
      Tuna tuna = getInstanceByType(Tuna.class);
      assert getCurrentConfiguration().getBeans().isProxy(tuna);
   }

   @Test
   @SpecAssertion(section = "5.3", id = "a")
   public void testClientProxyNotUsedForPseudoScope() throws Exception
   {
      new RunInDependentContext()
      {

         @Override
         protected void execute() throws Exception
         {
            Bean<Fox> foxBean = getBeans(Fox.class).iterator().next();
            Fox fox = (Fox) getCurrentManager().getReference(foxBean,Fox.class, getCurrentManager().createCreationalContext(foxBean));
            assert !getCurrentConfiguration().getBeans().isProxy(fox);
         }
         
      }.run();
   }
   
   @Test(groups="incontainer-ri-broken")
   //@SpecAssertion(section = "5.4", id = "c")
   @SpecAssertion(section = "unknown", id = "unknown")
   public void testSimpleWebBeanClientProxyIsSerializable() throws IOException, ClassNotFoundException
   {
      TunedTuna tuna = getInstanceByType(TunedTuna.class);
      assert getCurrentConfiguration().getBeans().isProxy(tuna);
      byte[] bytes = serialize(tuna);
      tuna = (TunedTuna) deserialize(bytes);
      assert getCurrentConfiguration().getBeans().isProxy(tuna);
      assert tuna.getState().equals("tuned");
   }

   @Test
   @SpecAssertion(section = "5.3.2", id = "aa")
   public void testClientProxyInvocation()
   {
      Bean<TunedTuna> tunaBean = getBeans(TunedTuna.class).iterator().next();
      TunedTuna tuna = (TunedTuna) getCurrentManager().getReference(tunaBean, TunedTuna.class, getCurrentManager().createCreationalContext(tunaBean));
      assert getCurrentConfiguration().getBeans().isProxy(tuna);
      assert tuna.getState().equals("tuned");
   }
   
}
