package org.jboss.jsr299.tck.tests.lookup.dependency.resolution;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.lang.reflect.Type;
import java.util.HashSet;
import java.util.Set;

import javax.enterprise.context.RequestScoped;
import javax.enterprise.inject.spi.Annotated;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.InjectionPoint;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.ForwardingBean;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Tests for dependency resolution problem detection
 * 
 * Spec version: 20090519
 */
@Artifact
public class DependencyResolutionTest extends AbstractJSR299Test
{
   class MockInjectionPoint implements InjectionPoint
   {
      private Bean<?> beanWithManager;
      private Type type;
      private Member member;
      private Set<Annotation> bindings;

      public MockInjectionPoint(Bean<?> bean, Type type, Member member, 
            Set<Annotation> bindings)
      {
         this.beanWithManager = bean;
         this.type = type;
         this.member = member;
         this.bindings = bindings;
      }

      public Bean<?> getBean()
      {
         return beanWithManager;
      }

      public Set<Annotation> getBindings()
      {
         return bindings;
      }

      public Member getMember()
      {
         return member;
      }

      public Type getType()
      {
         return type;
      }

      public Annotated getAnnotated()
      {
         // TODO Auto-generated method stub
         return null;
      }

      public boolean isDelegate()
      {
         // TODO Auto-generated method stub
         return false;
      }

      public boolean isTransient()
      {
         // TODO Auto-generated method stub
         return false;
      }      
   }

   @Test(groups = "rewrite")
   @SpecAssertion(section = "5.1.2", id = "aa")
   public void testUnsatisfiedDependencyDetected() throws Exception
   {
      Bean<Vanilla> vanillaBean = getBeans(Vanilla.class).iterator().next();

      Field injectedField = InjectedBean.class.getField("vanilla");
      Set<Annotation> bindings = new HashSet<Annotation>();
      bindings.add(new OtherLiteral());
      MockInjectionPoint injectionPoint = new MockInjectionPoint(vanillaBean, Vanilla.class, injectedField, bindings);

      assert getCurrentManager().getInjectableReference(injectionPoint, null) instanceof Vanilla;      
   }

   @Test(groups = "rewrite")
   @SpecAssertion(section = "5.1.2", id = "aa")
   public void testAmbiguousDependencyDetected() throws Exception
   {
      Bean<Vanilla> vanillaBean = getBeans(Vanilla.class).iterator().next();

      Field injectedField = InjectedBean.class.getField("vanilla");
      Set<Annotation> bindings = new HashSet<Annotation>();
      bindings.add(new OtherLiteral());
      MockInjectionPoint injectionPoint = new MockInjectionPoint(vanillaBean, Vanilla.class, injectedField, bindings);      
      getCurrentManager().getInjectableReference(injectionPoint, null);      
   }   

   @Test(groups = "rewrite")
   @SpecAssertion(section = "5.3.1", id = "aa")
   public void testUnproxyableDependencyDetected() throws Exception
   {
      final Bean<Mustard> bean = getBeans(Mustard.class).iterator().next();
      final Set<Annotation> bindings = new HashSet<Annotation>();
      bindings.add(new OtherLiteral());

      Bean<Mustard> wrappedBean = new ForwardingBean<Mustard>()
      {

         @Override
         protected Bean<Mustard> delegate()
         {
            return bean;
         }

         @Override
         public Class<? extends Annotation> getScopeType()
         {
            return RequestScoped.class;
         }

         @Override
         public Set<Annotation> getBindings()
         {
            return bindings;
         }

         @Override
         public boolean equals(Object obj)
         {
            return this == obj;
         }

      };

      getCurrentManager().addBean(wrappedBean);

      Field injectedField = BrokenInjectedBean.class.getField("mustard");
      MockInjectionPoint injectionPoint = new MockInjectionPoint(bean, Mustard.class, injectedField, bindings);
      getCurrentManager().getInjectableReference(injectionPoint, null);     
   }
}
