package org.jboss.jsr299.tck.tests.lookup.dynamic;

import static org.jboss.jsr299.tck.tests.lookup.dynamic.PayBy.PaymentMethod.CHEQUE;

import javax.enterprise.inject.Instance;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * Tests for dynamic lookup features
 * 
 * @author Shane Bryzak
 * 
 * Spec version: 20090519
 */
@Artifact
public class DynamicLookupTest extends AbstractJSR299Test
{
   @Test(groups = "ri-broken")
   @SpecAssertions({
      @SpecAssertion(section = "5.5", id ="aa")
   })
   public void testObtainsInjectsInstanceOfInstance()
   {
      Bean<ObtainsInjectionPointBean> injectionPointBean = 
         getBeans(ObtainsInjectionPointBean.class).iterator().next();
      
      ObtainsInjectionPointBean injectionPoint =  (ObtainsInjectionPointBean) 
    	  getCurrentManager().getReference(injectionPointBean,ObtainsInjectionPointBean.class, getCurrentManager().createCreationalContext(injectionPointBean));
      assert injectionPoint.getPaymentProcessor() instanceof Instance;
   }
   
   @Test(groups = "ri-broken")
   @SpecAssertions({
      @SpecAssertion(section = "5.5", id ="b"),
      @SpecAssertion(section = "5.5", id ="c"),
      @SpecAssertion(section = "5.5.1", id ="a")
   })
   public void testDynamicLookup()
   {
      Bean<ObtainsInjectionPointBean> injectionPointBean = 
         getBeans(ObtainsInjectionPointBean.class).iterator().next();
      
      ObtainsInjectionPointBean injectionPoint = (ObtainsInjectionPointBean) 
	      getCurrentManager().getReference( injectionPointBean, ObtainsInjectionPointBean.class, getCurrentManager().createCreationalContext(injectionPointBean));
      assert injectionPoint.getPaymentProcessor().get() instanceof ChequePaymentProcessor;      
   }   
   
   @Test(groups = "ri-broken", expectedExceptions = IllegalArgumentException.class)
   @SpecAssertion(section = "5.5.1", id ="d")   
   public void testDuplicateBindingsThrowsException()
   {
      Bean<ObtainsInjectionPointBean> injectionPointBean = 
         getBeans(ObtainsInjectionPointBean.class).iterator().next();
      
      ObtainsInjectionPointBean injectionPoint =  (ObtainsInjectionPointBean) 
    	  getCurrentManager().getReference(injectionPointBean,ObtainsInjectionPointBean.class, getCurrentManager().createCreationalContext(injectionPointBean));
      injectionPoint.getPaymentProcessor().get(new PayByBinding() { public PaymentMethod value() { return CHEQUE; }});      
   }      
   
   @Test(groups = "ri-broken", expectedExceptions = IllegalArgumentException.class)
   @SpecAssertion(section = "5.5.1", id = "e")
   public void testNonBindingThrowsException()
   {
      Bean<ObtainsInjectionPointBean> injectionPointBean = 
         getBeans(ObtainsInjectionPointBean.class).iterator().next();
      
      ObtainsInjectionPointBean injectionPoint =  (ObtainsInjectionPointBean) 
    	  getCurrentManager().getReference(injectionPointBean,ObtainsInjectionPointBean.class, getCurrentManager().createCreationalContext(injectionPointBean));
      injectionPoint.getPaymentProcessor().get(new NonBinding() );      
   }
   
}
