package org.jboss.jsr299.tck.tests.lookup.injection;

import org.hibernate.tck.annotations.SpecAssertion;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
@BeansXml("beans.xml")
public class InjectionTest extends AbstractJSR299Test
{

   @Test(groups = { "injection", "producerMethod" })
   @SpecAssertion(section = "5.1.3", id = "aa")
   public void testInjectionPerformsBoxingIfNecessary() throws Exception
   {
      assert getBeans(SpiderNest.class).size() == 1;
      SpiderNest spiderNest = getInstanceByType(SpiderNest.class);
      assert spiderNest.numberOfSpiders != null;
      assert spiderNest.numberOfSpiders.equals(4);
   }

   @Test 
   @SpecAssertion(section="4.2", id = "aa")
   public void testFieldDeclaredInSuperclassInjected() throws Exception
   {      
      new RunInDependentContext()
      {
        @Override
         protected void execute() throws Exception
         {
           DeluxeHenHouse henHouse = getInstanceByType(DeluxeHenHouse.class);
           assert henHouse.fox != null;
           assert henHouse.fox.getName().equals("gavin");
         }
      }.run();
   }
   
   @Test 
   @SpecAssertion(section="4.2", id = "ac")
   public void testFieldDeclaredInIndirectSuperclassInjected() throws Exception
   {
      new RunInDependentContext()
      {
        @Override
         protected void execute() throws Exception
         {
           MegaPoorHenHouse henHouse = getInstanceByType(MegaPoorHenHouse.class);
           assert henHouse.fox != null;
           assert henHouse.fox.getName().equals("gavin");
         }
      }.run();
   }

   @Test(groups = {"injection", "underInvestigation"})
   @SpecAssertion(section = "3.8", id = "aa")
   public void testInjectingStaticField()
   {
      assert getBeans(BeanWithStaticBoundField_Broken.class).size() == 1;
      assert getInstanceByType(BeanWithStaticBoundField_Broken.class).tuna == null;
   }

   @Test(groups = {"injection", "underInvestigation"})
   @SpecAssertion(section = "3.8", id = "ab")
   public void testInjectingFinalField()
   {
      assert getBeans(BeanWithFinalBoundField_Broken.class).size() == 1;
      assert getInstanceByType(BeanWithFinalBoundField_Broken.class).tuna == null;
   }

}
