package org.jboss.jsr299.tck.tests.lookup.manager;

import javax.enterprise.context.Dependent;
import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.Current;
import javax.enterprise.inject.deployment.Standard;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.testharness.impl.packaging.Artifact;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
public class ManagerTest extends AbstractJSR299Test
{
   @Test(groups={"manager", "injection", "deployment"}) 
   @SpecAssertion(section="11.2", id = "c")
   public void testInjectingManager() throws Exception
   {
      new RunInDependentContext()
      {
         
         @Override
         protected void execute() throws Exception
         {
            FishFarmOffice fishFarmOffice = getInstanceByType(FishFarmOffice.class);
            assert fishFarmOffice.beanManager != null;
         }
         
      }.run();      
   }  
   
   @Test
   @SpecAssertion(section = "11.2", id = "a")
   public void testContainerProvidesManagerBean()
   {
      assert getBeans(BeanManager.class).size() > 0;
   }
   
   @Test
   @SpecAssertion(section = "11.2", id = "a")
   public void testManagerBeanIsDependentScoped()
   {
      Bean<BeanManager> beanManager = getBeans(BeanManager.class).iterator().next();
      assert beanManager.getScopeType().equals(Dependent.class);
   }
   
   @Test
   @SpecAssertion(section = "11.2", id = "a")   
   public void testManagerBeanHasStandardDeployment()
   {
      Bean<BeanManager> beanManager = getBeans(BeanManager.class).iterator().next();      
      assert beanManager.getDeploymentType().equals(Standard.class);
   }

   @Test
   @SpecAssertion(section = "11.2", id = "a")
   public void testManagerBeanHasCurrentBinding()
   {      
      Bean<BeanManager> beanManager = getBeans(BeanManager.class).iterator().next();
      assert beanManager.getBindings().contains(new AnnotationLiteral<Current>(){});
   }
   
   @Test
   @SpecAssertions({
      @SpecAssertion(section = "11.2.1", id = "a"),
      @SpecAssertion(section = "11.2.1", id = "b")
   })
   public void testGetReferenceReturnsContextualInstance()
   {
      Bean<FishFarmOffice> bean = getBeans(FishFarmOffice.class).iterator().next();
      assert getCurrentManager().getReference(bean,FishFarmOffice.class, getCurrentManager().createCreationalContext(bean)) instanceof FishFarmOffice;      
   }
}
