package org.jboss.jsr299.tck.tests.lookup.typesafe.resolution;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.enterprise.inject.AnnotationLiteral;
import javax.enterprise.inject.TypeLiteral;
import javax.enterprise.inject.spi.Bean;

import org.hibernate.tck.annotations.SpecAssertion;
import org.hibernate.tck.annotations.SpecAssertions;
import org.jboss.jsr299.tck.AbstractJSR299Test;
import org.jboss.jsr299.tck.literals.CurrentLiteral;
import org.jboss.testharness.impl.packaging.Artifact;
import org.jboss.testharness.impl.packaging.jsr299.BeansXml;
import org.testng.annotations.Test;

/**
 * 
 * Spec version: 20090519
 *
 */
@Artifact
@BeansXml("beans.xml")
public class ResolutionByTypeTest extends AbstractJSR299Test
{

   @Test(groups="resolution")
   @SpecAssertions({
      @SpecAssertion(section="5.1", id = "gaa")
   })
   public void testDefaultBindingTypeAssumed() throws Exception
   {
      Set<Bean<Tuna>> possibleTargets = getBeans(Tuna.class);
      assert possibleTargets.size() == 1;
      assert possibleTargets.iterator().next().getTypes().contains(Tuna.class);
   }

   @Test(groups = { "resolution", "rewrite" }, expectedExceptions = IllegalArgumentException.class)
   // This assertion comes back later
   public  <T> void testParameterizedTypeWithTypeParameter()
   {
      getBeans(new TypeLiteral<ParameterizedBean_Broken<T>>(){});
   }

   @Test(groups = { "resolution", "rewrite" }, expectedExceptions=IllegalArgumentException.class)
   @SpecAssertion(section="5.1", id = "d")
   public void testDuplicateBindingTypesUsed()
   {
      getBeans(Tuna.class, new CurrentLiteral(), new CurrentLiteral());
   }

   @Test(groups="resolution", expectedExceptions=IllegalArgumentException.class)
   @SpecAssertion(section="5.1", id = "e")
   public void testNonBindingTypeUsed()
   {
      getBeans(Tuna.class, new AnotherDeploymentTypeLiteral());
   }

   @Test(groups="resolution")
   @SpecAssertions({
      @SpecAssertion(section = "5.1", id ="faa")
   })
   public void testResolveByType() throws Exception
   {

      assert getBeans(Tuna.class, new CurrentLiteral()).size() == 1;

      assert getBeans(Tuna.class).size() == 1;

      Set<Bean<Animal>> beans = getBeans(Animal.class, new AnnotationLiteral<FishILike>() {});
      assert beans.size() == 3;
      List<Class<? extends Animal>> classes = new ArrayList<Class<? extends Animal>>();
      for (Bean<Animal> bean : beans)
      {
         if (bean.getTypes().contains(Salmon.class))
         {
            classes.add(Salmon.class);
         }
         else if (bean.getTypes().contains(SeaBass.class))
         {
            classes.add(SeaBass.class);
         }
         else if (bean.getTypes().contains(Haddock.class))
         {
            classes.add(Haddock.class);
         }
      }
      assert classes.contains(Salmon.class);
      assert classes.contains(SeaBass.class);
      assert classes.contains(Haddock.class);
   }

   @Test(groups="injection")
   @SpecAssertions({
      @SpecAssertion(section = "2.3.4", id = "b" ),
      @SpecAssertion(section = "5.1",   id = "gba"),
      @SpecAssertion(section = "2.3.3", id = "d" ),
      @SpecAssertion(section = "5.1.5", id = "a" )
   })
   public void testAllBindingTypesSpecifiedForResolutionMustAppearOnWebBean()
   {
      assert getBeans(Animal.class, new ChunkyLiteral(), new AnnotationLiteral<Whitefish>() {}).size() == 1;
      assert getBeans(Animal.class, new ChunkyLiteral(), new AnnotationLiteral<Whitefish>() {}).iterator().next().getTypes().contains(Cod.class);

      assert getBeans(ScottishFish.class, new AnnotationLiteral<Whitefish>() {}).size() == 2;
      List<Class<? extends Animal>> classes = new ArrayList<Class<? extends Animal>>();
      for (Bean<ScottishFish> bean : getBeans(ScottishFish.class, new AnnotationLiteral<Whitefish>() {}))
      {
         if (bean.getTypes().contains(Cod.class))
         {
            classes.add(Cod.class);
         }
         else if (bean.getTypes().contains(Sole.class))
         {
            classes.add(Sole.class);
         }
      }
      assert classes.contains(Cod.class);
      assert classes.contains(Sole.class);
   }

   @Test(groups="resolution")
   @SpecAssertions({
      @SpecAssertion(section="5.1", id = "fd"),
      @SpecAssertion(section="2.2", id="bb"),
      @SpecAssertion(section="2.2", id="ba")
   })
   public void testResolveByTypeWithTypeParameter() throws Exception
   {
      assert getBeans(new TypeLiteral<Farmer<ScottishFish>>(){}).size() == 1;
      assert getBeans(new TypeLiteral<Farmer<ScottishFish>>(){}).iterator().next().getTypes().contains(ScottishFishFarmer.class);
   }

   @Test(groups={"resolution", "producerMethod"})
   @SpecAssertion(section="5.1", id = "fc")
   public void testResolveByTypeWithArray() throws Exception
   {
      assert getBeans(Spider[].class).size() == 1;
   }

   @Test @SpecAssertion(section="5.1", id = "ha")
   public void testOnlyHighestEnabledPrecedenceWebBeansResolved() throws Exception
   {
      assert getBeans(Animal.class, new AnnotationLiteral<Whitefish>() {}).size() == 1;
      assert getBeans(Animal.class, new AnnotationLiteral<Whitefish>() {}).iterator().next().getTypes().contains(Plaice.class);
   }


   @Test(groups="resolution")
   @SpecAssertion(section="5.1", id = "hb")
   public void testNoWebBeansFound() throws Exception
   {
      assert getBeans(Tuna.class, new AnnotationLiteral<FakeFish>() {}).size() == 0;
   }

   @Test(groups="resolution")
   @SpecAssertions({
      @SpecAssertion(section="5.1", id = "gca")
   })
   public void testResolveByTypeWithNonBindingMembers() throws Exception
   {

      Set<Bean<Animal>> beans = getBeans(Animal.class, new ExpensiveLiteral()
      {

         public int cost()
         {
            return 60;
         }

         public boolean veryExpensive()
         {
            return true;
         }

      }, new AnnotationLiteral<Whitefish>() {});
      assert beans.size() == 2;

      List<Class<? extends Animal>> classes = new ArrayList<Class<? extends Animal>>();
      for (Bean<Animal> bean : beans)
      {
         if (bean.getTypes().contains(Halibut.class))
         {
            classes.add(Halibut.class);
         }
         else if (bean.getTypes().contains(RoundWhitefish.class))
         {
            classes.add(RoundWhitefish.class);
         }
      }
      assert classes.contains(Halibut.class);
      assert classes.contains(RoundWhitefish.class);
   }

   @Test(groups={"resolution", "broken"})
   @SpecAssertion(section="5.1.4", id = "ca")
   public void testArrayValuedAnnotationMemberWithoutNonBinding()
   {
      Throwable t = null;
      try
      {
         getBeans(Animal.class, new BindingTypeWithBindingArrayTypeMemberLiteral_Broken()
         {

            public boolean[] bool()
            {
               return new boolean[0];
            }

         });
      }
      catch (Throwable throwable)
      {
         t = throwable;
      }

      //assert getCurrentConfiguration().getManagers().isDefinitionError(t);
   }

   @Test(groups={"resolution", "broken"})//, expectedExceptions=DefinitionException.class)
   @SpecAssertion(section="5.1.4", id = "cb")
   public void testAnnotationValuedAnnotationMemberWithoutNonBinding()
   {
      Throwable t = null;
      try
      {
         getBeans(Animal.class, new BindingTypeWithBindingAnnotationMemberLiteral_Broken()
         {
            public Expensive expensive()
            {
               return new ExpensiveLiteral()
               {
                  public int cost()
                  {
                     return 0;
                  }

                  public boolean veryExpensive()
                  {
                     return false;
                  }
               };
            }

         });
      }
      catch (Throwable throwable)
      {
         t = throwable;
      }

      //assert getCurrentConfiguration().getManagers().isDefinitionError(t);
   }

}
